@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2004, University of Amsterdam
+    Copyright (C): 1985-2009, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -30,65 +30,66 @@
 */
 
 :- module('$syspreds',
-	[ leash/1
-	, visible/1
-	, style_check/1
-	, (spy)/1
-	, (nospy)/1
-	, trace/1
-	, trace/2
-	, nospyall/0
-	, debugging/0
-	, rational/3
-	, concat_atom/2
-	, atom_prefix/2
-	, dwim_match/2
-	, source_file/1
-	, prolog_load_context/2
-	, stream_position_data/3
-	, current_predicate/2
-	, '$defined_predicate'/1
-	, predicate_property/2
-	, '$predicate_property'/2
-	, clause_property/2
-	, recorda/2
-	, recordz/2
-	, recorded/2
-	, current_module/1
-	, current_module/2
-	, module/1
-	, statistics/0
-	, shell/2
-	, shell/1
-	, shell/0
-	, on_signal/3
-	, current_signal/3
-	, open_shared_object/2
-	, open_shared_object/3
-	, format/1
-	, garbage_collect/0
-	, arithmetic_function/1
-        , default_module/2
-	, absolute_file_name/2
-	, require/1
-	, call_with_depth_limit/3
-	, length/2
-	, numbervars/3
-	, nb_setval/2				% +Var, +Value
-	]).	
+	  [ leash/1,
+	    visible/1,
+	    style_check/1,
+	    (spy)/1,
+	    (nospy)/1,
+	    trace/1,
+	    trace/2,
+	    nospyall/0,
+	    debugging/0,
+	    rational/3,
+	    atom_prefix/2,
+	    dwim_match/2,
+	    source_file/1,
+	    prolog_load_context/2,
+	    stream_position_data/3,
+	    current_predicate/2,
+	    '$defined_predicate'/1,
+	    predicate_property/2,
+	    '$predicate_property'/2,
+	    clause_property/2,
+	    recorda/2,
+	    recordz/2,
+	    current_module/1,
+	    module_property/2,
+	    module/1,
+	    statistics/0,
+	    shell/1,
+	    shell/0,
+	    on_signal/3,
+	    current_signal/3,
+	    open_shared_object/2,
+	    open_shared_object/3,
+	    format/1,
+	    garbage_collect/0,
+	    set_prolog_stack/2,
+	    prolog_stack_property/2,
+	    arithmetic_function/1,
+	    absolute_file_name/2,
+	    require/1,
+	    call_with_depth_limit/3,
+	    length/2,			% ?List, ?Length
+	    numbervars/3,		% +Term, +Start, -End
+	    nb_setval/2			% +Var, +Value
+	  ]).
 
 		/********************************
 		*           DEBUGGER            *
 		*********************************/
 
+:- meta_predicate
+	'$map_bits'(2, +, +, -).
+
 '$map_bits'(_, [], Bits, Bits) :- !.
 '$map_bits'(Pred, [H|T], Old, New) :-
 	'$map_bits'(Pred, H, Old, New0),
 	'$map_bits'(Pred, T, New0, New).
-'$map_bits'(Pred, +Name, Old, New) :- !, 		% set a bit
+'$map_bits'(Pred, +Name, Old, New) :- !,	% set a bit
 	call(Pred, Name, Bits), !,
 	New is Old \/ Bits.
-'$map_bits'(Pred, -Name, Old, New) :- !, 		% clear a bit
+'$map_bits'(Pred, -Name, Old, New) :- !, 	% clear a bit
 	call(Pred, Name, Bits), !,
 	New is Old /\ (\Bits).
 '$map_bits'(Pred, ?(Name), Old, Old) :-		% ask a bit
@@ -137,13 +138,6 @@ style_check(Spec) :-
 	'$map_bits'('$map_style_check', Spec, Old, New),
 	'$style_check'(_, New).
 
-:- module_transparent
-	trace/1,
-	trace/2,
-	'$trace'/2,
-	spy/1,
-	nospy/1.
-
 %	prolog:debug_control_hook(+Action)
 %
 %	Allow user-hooks in the Prolog debugger interaction.  See the calls
@@ -154,21 +148,33 @@ style_check(Spec) :-
 
 :- multifile
 	prolog:debug_control_hook/1.	% +Action
-:- module_transparent
-	prolog:debug_control_hook/1.
+
+%%	trace(:Preds) is det.
+%%	trace(:Preds, +PortSpec) is det.
+%
+%	Start printing messages if control passes specified ports of
+%	the given predicates.
+
+:- meta_predicate
+	trace(:),
+	trace(:, +).
 
 trace(Preds) :-
 	trace(Preds, +all).
 
-trace([], _) :- !.
-trace([H|T], Ps) :- !,
-	trace(H, Ps),
-	trace(T, Ps).
+trace(_:X, _) :-
+	var(X), !,
+	throw(error(instantiation_error, _)).
+trace(_:[], _) :- !.
+trace(M:[H|T], Ps) :- !,
+	trace(M:H, Ps),
+	trace(M:T, Ps).
 trace(Pred, Ports) :-
-	set_prolog_flag(debug, true),
 	'$find_predicate'(Pred, Preds),
 	Preds \== [],
-	(   '$member'(Head, Preds),
+	set_prolog_flag(debug, true),
+	(   '$member'(PI, Preds),
+	        pi_to_head(PI, Head),
 	        (   Head = _:_
 		->  QHead0 = Head
 		;   QHead0 = user:Head
@@ -219,29 +225,47 @@ tag_list([H0|T0], F, [H1|T1]) :-
 	H1 =.. [F, H0],
 	tag_list(T0, F, T1).
 
-spy([]) :- !.
-spy([H|T]) :- !,
-	spy(H),
-	spy(T).
+:- meta_predicate
+	spy(:),
+	nospy(:).
+
+%%	spy(:Spec) is det.
+%%	nospy(:Spec) is det.
+%%	nospyall is det.
+%
+%	Set/clear spy-points.
+
+spy(_:X) :-
+	var(X),
+	throw(error(instantiation_error, _)).
+spy(_:[]) :- !.
+spy(M:[H|T]) :- !,
+	spy(M:H),
+	spy(M:T).
 spy(Spec) :-
 	prolog:debug_control_hook(spy(Spec)), !.
 spy(Spec) :-
 	'$find_predicate'(Spec, Preds),
-	'$member'(Head, Preds),
+	'$member'(PI, Preds),
+	    pi_to_head(PI, Head),
 	    '$define_predicate'(Head),
 	    '$spy'(Head),
 	fail.
 spy(_).
 
-nospy([]) :- !.
-nospy([H|T]) :- !,
-	nospy(H),
-	nospy(T).
+nospy(_:X) :-
+	var(X),
+	throw(error(instantiation_error, _)).
+nospy(_:[]) :- !.
+nospy(M:[H|T]) :- !,
+	nospy(M:H),
+	nospy(M:T).
 nospy(Spec) :-
 	prolog:debug_control_hook(nospy(Spec)), !.
 nospy(Spec) :-
 	'$find_predicate'(Spec, Preds),
-	'$member'(Head, Preds),
+	'$member'(PI, Preds),
+	     pi_to_head(PI, Head),
 	    '$nospy'(Head),
 	fail.
 nospy(_).
@@ -255,6 +279,15 @@ nospyall :-
 	fail.
 nospyall.
 
+pi_to_head(M:PI, M:Head) :- !,
+	pi_to_head(PI, Head).
+pi_to_head(Name/Arity, Head) :-
+	functor(Head, Name, Arity).
+
+%%	debugging is det.
+%
+%	Report current status of the debugger.
+
 debugging :-
 	prolog:debug_control_hook(debugging), !.
 debugging :-
@@ -290,7 +323,7 @@ trace_ports(Head, Ports) :-
 		 *******************************/
 
 %	rational(+Rat, -M, -N)
-%	
+%
 %	Get parts of a rational number.
 
 rational(Rat, M, N) :-
@@ -307,11 +340,6 @@ rational(Rat, M, N) :-
 		*             ATOMS             *
 		*********************************/
 
-concat_atom([A, B], C) :- !,
-	atom_concat(A, B, C).
-concat_atom(L, Atom) :-
-	'$concat_atom'(L, Atom).
-
 dwim_match(A1, A2) :-
 	dwim_match(A1, A2, _).
 
@@ -330,15 +358,19 @@ atom_prefix(Atom, Prefix) :-
 %	bound to the canonical name for it. If File is bound it succeeds
 %	if the canonical name  as   defined  by  absolute_file_name/2 is
 %	known as a loaded filename.
+%
+%	Note that Time = 0.0 is used by  PlDoc and other code that needs
+%	to create a file record without being interested in the time.
 
 source_file(File) :-
 	(   ground(File)
-	->  ( 	'$time_source_file'(File, _, user)
+	->  ( 	'$time_source_file'(File, Time, user)
 	    ;	absolute_file_name(File, Abs),
-		'$time_source_file'(Abs, _, user)
+		'$time_source_file'(Abs, Time, user)
 	    ), !
-	;   '$time_source_file'(File, _, user)
-	).
+	;   '$time_source_file'(File, Time, user)
+	),
+	Time > 0.0.
 
 %%	prolog_load_context(+Key, -Value)
 %
@@ -346,9 +378,6 @@ source_file(File) :-
 %	Note  that  only  the  line-number  info    is   valid  for  the
 %	'$stream_position'. Largely Quintus compatible.
 
-:- module_transparent
-	prolog_load_context/2.
-
 prolog_load_context(module, Module) :-
 	'$set_source_module'(Module, Module).
 prolog_load_context(file, F) :-
@@ -364,16 +393,23 @@ prolog_load_context(directory, D) :-
 	source_location(F, _),
 	file_directory_name(F, D).
 prolog_load_context(dialect, D) :-
-	'$set_dialect'(D, D).
+	current_prolog_flag(emulated_dialect, D).
 prolog_load_context(term_position, '$stream_position'(0,L,0,0,0)) :-
 	source_location(_, L).
+prolog_load_context(script, Bool) :-
+	(   '$toplevel':loaded_init_file(script, Path),
+	    source_location(Path, _)
+	->  Bool = true
+	;   Bool = false
+	).
+
 
 		 /*******************************
 		 *	      STREAMS		*
 		 *******************************/
 
 %%	stream_position_data(?Field, +Pos, ?Date)
-%	
+%
 %	Extract values from stream position objects. '$stream_position' is
 %	of the format '$stream_position'(Byte, Char, Line, LinePos)
 
@@ -397,13 +433,14 @@ stream_position_field(byte_count,    4).
 		 *	      CONTROL		*
 		 *******************************/
 
-%	call_with_depth_limit(+Goal, +DepthLimit, -Result)
+%%	call_with_depth_limit(:Goal, +DepthLimit, -Result)
 %
 %	Try to proof Goal, but fail on any branch exceeding the indicated
 %	depth-limit.  Unify Result with the maximum-reached limit on success,
 %	depth_limit_exceeded if the limit was exceeded and fails otherwise.
 
-:- module_transparent call_with_depth_limit/3.
+:- meta_predicate
+	call_with_depth_limit(0, +, -).
 
 call_with_depth_limit(G, Limit, Result) :-
 	'$depth_limit'(Limit, OLimit, OReached),
@@ -432,58 +469,70 @@ etc.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 
-:- module_transparent
-	current_predicate/2,
-	'$defined_predicate'/1.
+:- meta_predicate
+	current_predicate(?, :),
+	'$defined_predicate'(:).
 
-current_predicate(Name, Head) :-
-	var(Head), !,
-	context_module(Module),
-	generate_current_predicate(Name, Module, Head).
 current_predicate(Name, Module:Head) :-
 	(var(Module) ; var(Head)), !,
 	generate_current_predicate(Name, Module, Head).
 current_predicate(Name, Term) :-
 	'$c_current_predicate'(Name, Term),
 	'$defined_predicate'(Term), !.
-current_predicate(Name, Term) :-
-	strip_module(Term, Module, Head),
+current_predicate(Name, Module:Head) :-
 	default_module(Module, DefModule),
 	'$c_current_predicate'(Name, DefModule:Head),
 	'$defined_predicate'(DefModule:Head), !.
-current_predicate(Name, Term) :-
+current_predicate(Name, Module:Head) :-
 	current_prolog_flag(autoload, true),
-	strip_module(Term, Module, Head),
+	\+ current_prolog_flag(Module:unknown, fail),
 	functor(Head, Name, Arity),
 	'$find_library'(Module, Name, Arity, _LoadModule, _Library), !.
 
 generate_current_predicate(Name, Module, Head) :-
 	current_module(Module),
-	'$c_current_predicate'(Name, Module:Head),
-	'$defined_predicate'(Module:Head).
+	QHead = Module:Head,
+	'$c_current_predicate'(Name, QHead),
+	'$get_predicate_attribute'(QHead, defined, 1).
 
 '$defined_predicate'(Head) :-
 	'$get_predicate_attribute'(Head, defined, 1), !.
 
-:- module_transparent
-	predicate_property/2,
-	'$predicate_property'/2.
+:- meta_predicate
+	predicate_property(:, ?).
 
 :- '$iso'(predicate_property/2).
 
 predicate_property(Pred, Property) :-
 	Property == undefined, !,
-	(   Pred = Module:Head,
-	    var(Module)
-	;   strip_module(Pred, Module, Head)
-	), !,
+	Pred = Module:Head,
 	current_module(Module),
-	Term = Module:Head,
-	'$c_current_predicate'(_, Term),
-	\+ '$defined_predicate'(Term),		% Speed up a bit
-	\+ current_predicate(_, Term).
+	'$c_current_predicate'(_, Pred),
+	\+ '$defined_predicate'(Pred),		% Speed up a bit
+	\+ current_predicate(_, Pred),
+	functor(Head, Name, Arity),
+	\+ system_undefined(Module:Name/Arity).
+predicate_property(_:Head, Property) :-
+	nonvar(Property),
+	Property = autoload(File), !,
+	current_prolog_flag(autoload, true),
+	(   callable(Head)
+	->  functor(Head, Name, Arity),
+	    (	'$find_library'(_, Name, Arity, _, File)
+	    ->	true
+	    )
+	;   '$find_library'(_, Name, Arity, _, File),
+	    functor(Head, Name, Arity)
+	).
+predicate_property(Pred, Property) :-
+	Pred = M:_,
+	M == system, !,				% do not autoload into system
+	'$c_current_predicate'(_, Pred),
+	'$defined_predicate'(Pred),
+	'$predicate_property'(Property, Pred).
 predicate_property(Pred, Property) :-
 	current_predicate(_, Pred),
+	'$define_predicate'(Pred),		% autoload if needed
 	'$predicate_property'(Property, Pred).
 
 '$predicate_property'(interpreted, Pred) :-
@@ -508,6 +557,8 @@ predicate_property(Pred, Property) :-
 	'$get_predicate_attribute'(Pred, transparent, 1).
 '$predicate_property'(indexed(Pattern), Pred) :-
 	'$get_predicate_attribute'(Pred, indexed, Pattern).
+'$predicate_property'(meta_predicate(Pattern), Pred) :-
+	'$get_predicate_attribute'(Pred, meta_predicate, Pattern).
 '$predicate_property'(file(File), Pred) :-
 	source_file(Pred, File).
 '$predicate_property'(line_count(LineNumber), Pred) :-
@@ -531,38 +582,64 @@ predicate_property(Pred, Property) :-
 '$predicate_property'(iso, Pred) :-
 	'$get_predicate_attribute'(Pred, iso, 1).
 
-:- index(clause_property(0, 1)).
+system_undefined(user:prolog_trace_interception/4).
+system_undefined(user:prolog_exception_hook/4).
+system_undefined(system:'$c_call_prolog'/0).
+system_undefined(system:window_title/2).
 
-clause_property(Clause, line_count(LineNumber)) :-
+%%	clause_property(+ClauseRef, ?Property) is nondet.
+%
+%	Provide information on individual clauses.  Defined properties
+%	are:
+%
+%	    * line_count(-Line)
+%	    Line from which the clause is loaded.
+%	    * file(-File)
+%	    File from which the clause is loaded.
+%	    * fact
+%	    Clause has body =true=.
+%	    * erased
+%	    Clause was erased.
+
+clause_property(Clause, Property) :-
+	'$clause_property'(Property, Clause).
+
+'$clause_property'(line_count(LineNumber), Clause) :-
 	'$get_clause_attribute'(Clause, line_count, LineNumber).
-clause_property(Clause, file(File)) :-
+'$clause_property'(file(File), Clause) :-
 	'$get_clause_attribute'(Clause, file, File).
-clause_property(Clause, fact) :-
+'$clause_property'(fact, Clause) :-
 	'$get_clause_attribute'(Clause, fact, true).
-clause_property(Clause, erased) :-
+'$clause_property'(erased, Clause) :-
 	'$get_clause_attribute'(Clause, erased, true).
 
 
-recorda(Key, Value) :-
-	recorda(Key, Value, _).
-recordz(Key, Value) :-
-	recordz(Key, Value, _).
-recorded(Key, Value) :-
-	recorded(Key, Value, _).
-
 		 /*******************************
 		 *	       REQUIRE		*
 		 *******************************/
 
-:- module_transparent
-	require/1.
+:- meta_predicate
+	require(:).
 
-require([]).
-require([N/A|T]) :- !,
+%%	require(:ListOfPredIndicators) is det.
+%
+%	Tag given predicates as undefined, so they will be included
+%	into a saved state through the autoloader.
+%
+%	@see autoload/0.
+
+require(M:List) :-
+	(   is_list(List)
+	->  require(List, M)
+	;   throw(error(type_error(list, List), _))
+	).
+
+require([], _).
+require([N/A|T], M) :- !,
 	functor(Head, N, A),
-	'$require'(Head),
-	require(T).
-require([H|_T]) :-
+	'$require'(M:Head),
+	require(T, M).
+require([H|_T], _) :-
 	throw(error(type_error(predicate_indicator, H), _)).
 
 
@@ -570,12 +647,44 @@ require([H|_T]) :-
 		*            MODULES            *
 		*********************************/
 
+%%	current_module(?Module) is nondet.
+%
+%	True if Module is a currently defined module.
+
 current_module(Module) :-
 	'$current_module'(Module, _).
 
-current_module(Module, File) :-
+%%	module_property(?Module, ?Property) is nondet.
+%
+%	True if Property is a property of Module.  Defined properties
+%	are:
+%
+%	    * file(File)
+%	    Module is loaded from File.
+%	    * line_count(Count)
+%	    The module declaration is on line Count of File.
+%	    * exports(ListOfPredicateIndicators)
+%	    The module exports ListOfPredicateIndicators
+
+module_property(Module, Property) :-
+	nonvar(Module), nonvar(Property), !,
+	'$module_property'(Module, Property).
+module_property(Module, Property) :-
+	nonvar(Property), Property = file(File), !,
 	'$current_module'(Module, File),
 	File \== [].
+module_property(Module, Property) :-
+	current_module(Module),
+	module_property(Property),
+	'$module_property'(Module, Property).
+
+module_property(file(_)).
+module_property(line_count(_)).
+module_property(exports(_)).
+
+%%	module(+Module) is det.
+%
+%	Set the module that is associated to the toplevel to Module.
 
 module(Module) :-
 	atom(Module),
@@ -590,6 +699,9 @@ module(Module) :-
 		*********************************/
 
 statistics :-
+	statistics(user_error).
+
+statistics(Out) :-
 	statistics(trail, Trail),
 	statistics(trailused, TrailUsed),
 	statistics(local, Local),
@@ -608,50 +720,50 @@ statistics :-
 	statistics(globallimit, GlobalLimit),
 	statistics(traillimit, TrailLimit),
 
-	format('~2f seconds cpu time for ~D inferences~n',
+	format(Out, '~2f seconds cpu time for ~D inferences~n',
 				    [Cputime, Inferences]),
-	format('~D atoms, ~D functors, ~D predicates, ~D modules, ~D VM-codes~n~n',
+	format(Out, '~D atoms, ~D functors, ~D predicates, ~D modules, ~D VM-codes~n~n',
 				    [Atoms, Functors, Predicates, Modules, Codes]),
-	format('                       Limit    Allocated       In use~n'),
+	format(Out, '                       Limit    Allocated       In use~n', []),
 	(   statistics(heap, Heap),
 	    statistics(heaplimit, HeapLimit)
-	->  format('Heap         :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
+	->  format(Out, 'Heap         :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
 		   [HeapLimit, Heap, Heapused])
-	;   format('Heap         :                  ~t~D~54| Bytes~n',
+	;   format(Out, 'Heap         :                  ~t~D~54| Bytes~n',
 		   [Heapused])
 	),
-	format('Local  stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
+	format(Out, 'Local  stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
 	       [LocalLimit, Local, LocalUsed]),
-	format('Global stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
+	format(Out, 'Global stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n',
 	       [GlobalLimit, Global, GlobalUsed]),
-	format('Trail  stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n~n',
+	format(Out, 'Trail  stack :~t~D~28| ~t~D~41| ~t~D~54| Bytes~n~n',
 	       [TrailLimit, Trail, TrailUsed]),
 
-	gc_statistics,
-	agc_statistics,
-	shift_statistics,
-	thread_statistics.
+	gc_statistics(Out),
+	agc_statistics(Out),
+	shift_statistics(Out),
+	thread_statistics(Out).
 
-gc_statistics :-
+gc_statistics(Out) :-
 	statistics(collections, Collections),
 	Collections > 0, !,
 	statistics(collected, Collected),
 	statistics(gctime, GcTime),
 
-	format('~D garbage collections gained ~D bytes in ~2f seconds.~n',
+	format(Out, '~D garbage collections gained ~D bytes in ~2f seconds.~n',
 	       [Collections, Collected, GcTime]).
-gc_statistics.
+gc_statistics(_).
 
-agc_statistics :-
+agc_statistics(Out) :-
 	catch(statistics(agc, Agc), _, fail),
 	Agc > 0, !,
 	statistics(agc_gained, Gained),
 	statistics(agc_time, Time),
-	format('~D atom garbage collections gained ~D atoms in ~2f seconds.~n',
+	format(Out, '~D atom garbage collections gained ~D atoms in ~2f seconds.~n',
 	       [Agc, Gained, Time]).
-agc_statistics.
+agc_statistics(_).
 
-shift_statistics :-
+shift_statistics(Out) :-
 	statistics(local_shifts, LS),
 	statistics(global_shifts, GS),
 	statistics(trail_shifts, TS),
@@ -659,28 +771,25 @@ shift_statistics :-
 	;   GS > 0
 	;   TS > 0
 	), !,
-	format('Stack shifts: ~D local, ~D global, ~D trail.~n',
+	format(Out, 'Stack shifts: ~D local, ~D global, ~D trail.~n',
 	       [LS, GS, TS]).
-shift_statistics.
+shift_statistics(_).
 
-thread_statistics :-
+thread_statistics(Out) :-
 	current_prolog_flag(threads, true), !,
 	statistics(threads, Active),
 	statistics(threads_created, Created),
 	statistics(thread_cputime, CpuTime),
 	Finished is Created - Active,
-	format('~D threads, ~D finished threads used ~2f seconds.~n',
+	format(Out, '~D threads, ~D finished threads used ~2f seconds.~n',
 	       [Active, Finished, CpuTime]).
-thread_statistics.
+thread_statistics(_).
 
 
 		/********************************
 		*      SYSTEM INTERACTION       *
 		*********************************/
 
-shell(Command, Status) :-
-	'$shell'(Command, Status).
-
 shell(Command) :-
 	shell(Command, 0).
 
@@ -697,8 +806,9 @@ shell :-
 		 *	      SIGNALS		*
 		 *******************************/
 
-:- module_transparent
-	on_signal/3.
+:- meta_predicate
+	on_signal(+, :, :),
+	current_signal(?, ?, :).
 
 on_signal(Signal, Old, New) :-
 	atom(Signal), !,
@@ -722,9 +832,28 @@ current_signal(Name, Id, Handler) :-
 		 *	      DLOPEN		*
 		 *******************************/
 
-:- module_transparent
-	open_shared_object/2,
-	open_shared_object/3.
+%%	open_shared_object(+File, -Handle) is det.
+%%	open_shared_object(+File, -Handle, +Flags) is det.
+%
+%	Open a shared object or DLL file. Flags  is a list of flags. The
+%	following flags are recognised. Note   however  that these flags
+%	may have no affect on the target platform.
+%
+%	    * =now=
+%	    Resolve all symbols in the file now instead of lazily.
+%	    * =global=
+%	    Make new symbols globally known.
+
+open_shared_object(File, Handle) :-
+	open_shared_object(File, Handle, []). % use pl-load.c defaults
+
+open_shared_object(File, Handle, Flags) :-
+	(   is_list(Flags)
+	->  true
+	;   throw(error(type_error(list, Flags), _))
+	),
+	map_dlflags(Flags, Mask),
+	'$open_shared_object'(File, Handle, Mask).
 
 dlopen_flag(now,	2'01).		% see pl-load.c for these constants
 dlopen_flag(global,	2'10).		% Solaris only
@@ -732,19 +861,12 @@ dlopen_flag(global,	2'10).		% Solaris only
 map_dlflags([], 0).
 map_dlflags([F|T], M) :-
 	map_dlflags(T, M0),
-	dlopen_flag(F, I),
+	(   dlopen_flag(F, I)
+	->  true
+	;   throw(error(domain_error(dlopen_flag, F), _))
+	),
 	M is M0 \/ I.
 
-open_shared_object(File, Flags, Handle) :- % compatibility
-	is_list(Flags), \+ is_list(Handle), !,
-	open_shared_object(File, Handle, Flags).
-open_shared_object(File, Handle, Flags) :-
-	map_dlflags(Flags, Mask),
-	'$open_shared_object'(File, Handle, Mask).
-
-open_shared_object(File, Handle) :-
-	open_shared_object(File, Handle, []). % use pl-load.c defaults
-
 
 		 /*******************************
 		 *	       I/O		*
@@ -763,33 +885,68 @@ absolute_file_name(Name, Abs) :-
 	atomic(Name), !,
 	'$absolute_file_name'(Name, Abs).
 absolute_file_name(Term, Abs) :-
-	'$chk_file'(Term, [''], [access(read)], File), !,
+	'$chk_file'(Term, [''], [access(read)], true, File), !,
 	'$absolute_file_name'(File, Abs).
 absolute_file_name(Term, Abs) :-
-	'$chk_file'(Term, [''], [], File), !,
+	'$chk_file'(Term, [''], [], true, File), !,
 	'$absolute_file_name'(File, Abs).
 
 
 		/********************************
-		*         MISCELLENEOUS         *
+		*	 MEMORY MANAGEMENT      *
 		*********************************/
 
-%	Invoke the garbage collector.  The argument is the debugging level
-%	to use during garbage collection.  This only works if the system
-%	is compiled with the -DODEBUG cpp flag.  Only to simplify maintenance.
+%%	garbage_collect is det.
+%
+%	Invoke the garbage collector.  The   argument  of the underlying
+%	'$garbage_collect'/1  is  the  debugging  level  to  use  during
+%	garbage collection. This only works if   the  system is compiled
+%	with the -DODEBUG cpp flag. Only to simplify maintenance.
 
 garbage_collect :-
 	'$garbage_collect'(0).
 
-%	arithmetic_function(Spec)
+%%	set_prolog_stack(+Name, +Option) is det.
+%
+%	Set a parameter for one of the Prolog stacks.
+
+set_prolog_stack(Stack, Option) :-
+	Option =.. [Name,Value0],
+	Value is Value0,
+	'$set_prolog_stack'(Stack, Name, _Old, Value).
+
+%%	prolog_stack_property(?Stack, ?Property) is nondet.
+%
+%	Examine stack properties.
+
+prolog_stack_property(Stack, Property) :-
+	stack_property(P),
+	stack_name(Stack),
+	Property =.. [P,Value],
+	'$set_prolog_stack'(Stack, P, Value, Value).
+
+stack_name(local).
+stack_name(global).
+stack_name(trail).
+
+stack_property(limit).
+stack_property(spare).
+stack_property(min_free).
+
+
+		 /*******************************
+		 *	     ARITHMETIC		*
+		 *******************************/
+
+%%	arithmetic_function(:Spec)
+%
 %	Register a predicate as an arithmetic function.  Takes Name/Arity
 %	and a term as argument.
 
-:- module_transparent
-	arithmetic_function/1.
+:- meta_predicate
+	arithmetic_function(1).
 
-arithmetic_function(Spec) :-
-	strip_module(Spec, Module, Term),
+arithmetic_function(Module:Term) :-
 	(   Term = Name/Arity
 	;   functor(Term, Name, Arity)
 	), !,
@@ -797,15 +954,6 @@ arithmetic_function(Spec) :-
 	functor(Head, Name, PredArity),
 	'$arithmetic_function'(Module:Head, 0).
 
-%	default_module(+Me, -Super)
-%	Is true if `Super' is `Me' or a super (auto import) module of `Me'.
-
-default_module(Me, Me).
-default_module(Me, Super) :-
-	import_module(Me, S),
-	default_module(S, Super).
-
-
 		 /*******************************
 		 *	 LIST MANIPULATION	*
 		 *******************************/
@@ -842,7 +990,7 @@ length3([_|List], N, N0) :-
 :- '$iso'((numbervars/3)).
 
 %	numbervars(+Term, +StartIndex, -EndIndex)
-%	
+%
 %	Number all unbound variables in Term   using  '$VAR'(N), where the
 %	first N is StartIndex and EndIndex is  unified to the index that
 %	will be given to the next variable.
@@ -856,7 +1004,7 @@ numbervars(Term, From, To) :-
 		 *******************************/
 
 %	nb_setval(+Name, +Value)
-%	
+%
 %	Bind the non-backtrackable variable Name with a copy of Value
 
 nb_setval(Name, Value) :-
