@@ -30,34 +30,36 @@
 */
 
 :- module('$toplevel',
-	[ '$initialise'/0		% start Prolog (does not return)
-	, '$toplevel'/0			% Prolog top-level (re-entrant)
-	, '$abort'/0 			% restart after an abort
-	, '$break'/0 			% live in a break
-	, '$compile'/0 			% `-c' toplevel
-	, '$welcome'/0			% banner
-	, prolog/0 			% user toplevel predicate
-	, '$set_prompt'/1			% set the main prompt
-	, at_initialization/1		% goals to run at initialization
-	, (initialization)/1		% initialization goal (directive)
-	, '$thread_init'/0		% initialise thread
-	, (thread_initialization)/1	% thread initialization goal
-	]).
+	  [ '$initialise'/0,		% start Prolog (does not return)
+	    '$toplevel'/0,		% Prolog top-level (re-entrant)
+	    '$abort'/0,			% restart after an abort
+	    '$break'/0,			% live in a break
+	    '$compile'/0,		% `-c' toplevel
+	    '$welcome'/0,		% banner
+	    prolog/0, 			% user toplevel predicate
+	    '$set_prompt'/1,		% set the main prompt
+	    (initialization)/1,		% initialization goal (directive)
+	    '$thread_init'/0,		% initialise thread
+	    (thread_initialization)/1	% thread initialization goal
+	    ]).
 
 
 		/********************************
 		*         INITIALISATION        *
 		*********************************/
 
+%	note: loaded_init_file/2 is used by prolog_load_context/2 to
+%	confirm we are loading a script.
+
 :- dynamic
-	loaded_init_file/1.		% already loaded init files
+	loaded_init_file/2.		% already loaded init files
 
 '$welcome' :-
 	print_message(banner, welcome).
 
 '$load_init_file'(none) :- !.
 '$load_init_file'(Base) :-
-	loaded_init_file(Base), !.
+	loaded_init_file(Base, _), !.
 '$load_init_file'(InitFile) :-
 	is_absolute_file_name(InitFile), !,
 	ensure_loaded(user:InitFile).
@@ -66,34 +68,40 @@
 			   [ access(read),
 			     file_errors(fail)
 			   ], InitFile),
-	asserta(loaded_init_file(Base)),
+	asserta(loaded_init_file(Base, InitFile)),
 	ensure_loaded(user:InitFile).
 '$load_init_file'(_).
 
 '$load_system_init_file' :-
-	loaded_init_file(system), !.
+	loaded_init_file(system, _), !.
 '$load_system_init_file' :-
 	'$option'(system_init_file, Base, Base),
-	(   Base == none
-	->  asserta(loaded_init_file(system))
-	;   current_prolog_flag(home, Home),
-	    file_name_extension(Base, rc, Name),
-	    concat_atom([Home, '/', Name], File),
-	    access_file(File, read),
-	    asserta(loaded_init_file(system)),
-	    load_files(user:File, [silent(true)]), !
-	).
+	Base \== none,
+	current_prolog_flag(home, Home),
+	file_name_extension(Base, rc, Name),
+	atomic_list_concat([Home, '/', Name], File),
+	absolute_file_name(File, Path,
+			   [ file_type(prolog),
+			     access(read),
+			     file_errors(fail)
+			   ]),
+	asserta(loaded_init_file(system, Path)),
+	load_files(user:Path, [silent(true)]), !.
 '$load_system_init_file'.
 
 '$load_script_file' :-
-	loaded_init_file(script), !.
+	loaded_init_file(script, _), !.
 '$load_script_file' :-
 	'$option'(script_file, OsFile, OsFile),
 	OsFile \== '',
 	prolog_to_os_filename(File, OsFile),
-	(   exists_file(File)		% avoid expanding on extensions
-	->  asserta(loaded_init_file(script)),
-	    load_files(user:File, [expand(false)])
+	(   absolute_file_name(File, Path,
+			       [ file_type(prolog),
+				 access(read),
+				 file_errors(fail)
+			       ])
+	->  asserta(loaded_init_file(script, Path)),
+	    load_files(user:Path, [])
 	;   throw(error(existence_error(script_file, File), _))
 	).
 '$load_script_file'.
@@ -111,57 +119,69 @@
 		 *	 AT_INITIALISATION	*
 		 *******************************/
 
-:- module_transparent
-	at_initialization/1,
-	(initialization)/1.
-:- dynamic
-	'$at_initialization'/1.
+:- meta_predicate
+	initialization(0).
 
-at_initialization(Spec) :-
-	strip_module(Spec, Module, Goal),
-	'$toplevel':assert('$at_initialization'(Module:Goal)).
+:- '$iso'((initialization)/1).
 
-'$run_at_initialization' :-
-	\+ current_prolog_flag(saved_program, true), !.
-'$run_at_initialization' :-
-	(   '$at_initialization'(Goal),
-	    (   catch(Goal, E,
-		      print_message(error, initialization_exception(Goal, E)))
-	    ->  fail
-	    ;   print_message(warning, goal_failed(at_initialization, Goal)),
-		fail
-	    )
-	;   true
-	),
-	'$thread_init'.
-
-%	initialization(+Goal)
+%%	initialization(:Goal)
 %
-%	Runs `Goal' both a load and initialization time.
+%	Runs Goal after loading the file in which this directive
+%	appears as well as after restoring a saved state.
+%
+%	@see initialization/2
 
 initialization(Goal) :-
-	at_initialization(Goal),
-	Goal.
+	Goal = _:G,
+	prolog:initialize_now(G, Use), !,
+	print_message(warning, initialize_now(G, Use)),
+	initialization(Goal, now).
+initialization(Goal) :-
+	initialization(Goal, after_load).
+
+:- multifile
+	prolog:initialization_now/2,
+	prolog:message//1.
+
+prolog:initialize_now(load_foreign_library(_),
+		      'use :- use_foreign_library/1 instead').
+prolog:initialize_now(load_foreign_library(_,_),
+		      'use :- use_foreign_library/2 instead').
+
+prolog:message(initialize_now(Goal, Use)) -->
+	[ 'Initialization goal ~p will be executed'-[Goal],nl,
+	  'immediately for backward compatibility reasons', nl,
+	  '~w'-[Use]
+	].
+
+'$run_initialization' :-
+	'$run_initialization'(_),
+	'$thread_init'.
 
 
 		 /*******************************
 		 *     THREAD INITIALIZATION	*
 		 *******************************/
 
-:- module_transparent
-	(thread_initialization)/1.
+:- meta_predicate
+	thread_initialization(0).
 :- dynamic
 	'$at_thread_initialization'/1.
 
-thread_initialization(Spec) :-
-	strip_module(Spec, Module, Goal),
-	'$toplevel':assert('$at_thread_initialization'(Module:Goal)),
-	Spec.
+%%	thread_initialization(:Goal)
+%
+%	Run Goal now and everytime a new thread is created.
+
+thread_initialization(Goal) :-
+	assert('$at_thread_initialization'(Goal)),
+	call(Goal), !.
 
 '$thread_init' :-
 	(   '$at_thread_initialization'(Goal),
-	    Goal,
-	    fail
+	    (	call(Goal)
+	    ->	fail
+	    ;	fail
+	    )
 	;   true
 	).
 
@@ -228,7 +248,7 @@ path_sep -->
 		 *******************************/
 
 %%	set_associated_file
-%	
+%
 %	If SWI-Prolog is started as <exe> <file>.<ext>, where <ext> is
 %	the extension registered for associated files, set the Prolog
 %	flag associated_file, switch to the directory holding the file
@@ -248,7 +268,7 @@ set_associated_file :-
 	access_file(File, read), !,
 	file_directory_name(File, Dir),
 	working_directory(_, Dir),
-	set_prolog_flag(associated_file, File),
+	create_prolog_flag(associated_file, File, []),
 	(   current_predicate(system:window_title/2)
 	->  atom_concat('SWI-Prolog -- ', File, Title),
 	    system:window_title(_, Title)
@@ -259,7 +279,7 @@ set_associated_file.
 
 %%	start_pldoc
 %
-%	If the option  =|--pldoc[=port]|=  is   given,  load  the PldDoc
+%	If the option  =|--pldoc[=port]|=  is   given,  load  the  PlDoc
 %	system.
 
 start_pldoc :-
@@ -282,7 +302,7 @@ start_pldoc.
 
 
 %%	load_associated_file
-%	
+%
 %	Load  the  file-name  set  by   set_associated_file/0  from  the
 %	commandline arguments. Note the expand(false) to avoid expanding
 %	special characters in the filename.
@@ -305,7 +325,7 @@ hkey('HKEY_LOCAL_MACHINE/Software/SWI/Prolog').
 	->  true
 	;   Ext = Ext0
 	),
-	set_prolog_flag(associate, Ext).
+	create_prolog_flag(associate, Ext, []).
 '$set_prolog_file_extension'.
 
 
@@ -328,11 +348,11 @@ initialise_prolog :-
 	set_associated_file,
 	'$set_file_search_paths',
 	once(print_predicate(_, [print], PrintOptions)),
-	set_prolog_flag(toplevel_print_options, PrintOptions),
-	set_prolog_flag(prompt_alternatives_on, determinism),
-	set_prolog_flag(toplevel_extra_white_line, true),
+	create_prolog_flag(toplevel_print_options, PrintOptions, []),
+	create_prolog_flag(prompt_alternatives_on, determinism, []),
+	create_prolog_flag(toplevel_extra_white_line, true, []),
 	'$set_debugger_print_options'(print),
-	'$run_at_initialization',
+	'$run_initialization',
 	'$load_system_init_file',
 	'$load_gnu_emacs_interface',
 	'$option'(init_file, OsFile, OsFile),
@@ -341,8 +361,8 @@ initialise_prolog :-
 	start_pldoc,
 	'$load_script_file',
 	load_associated_file,
-	'$option'(goal, GoalAtom, GoalAtom), 
-	term_to_atom(Goal, GoalAtom), 
+	'$option'(goal, GoalAtom, GoalAtom),
+	term_to_atom(Goal, GoalAtom),
 	(   Goal == '$welcome'
 	->  flag('$banner_goal', TheGoal, TheGoal)
 	;   TheGoal = Goal
@@ -350,24 +370,24 @@ initialise_prolog :-
 	ignore(user:TheGoal).
 
 '$abort' :-
-	see(user), 
-	tell(user), 
-	flag('$break_level', _, 0), 
+	see(user),
+	tell(user),
+	flag('$break_level', _, 0),
 	flag('$compilation_level', _, 0),
 	'$calleventhook'(abort),
 	print_message(informational, '$aborted'),
 	'$toplevel'.
 
 '$break' :-
-	flag('$break_level', Old, Old+1), 
-	flag('$break_level', New, New), 
+	flag('$break_level', Old, Old+1),
+	flag('$break_level', New, New),
 	print_message(informational, break(enter(New))),
 	'$runtoplevel',
 	print_message(informational, break(exit(New))),
 	flag('$break_level', _, Old), !.
 
-:- '$hide'('$toplevel', 0).			% avoid in the GUI stacktrace
-:- '$hide'('$abort', 0).			% same after an abort
+:- '$hide'('$toplevel'/0).		% avoid in the GUI stacktrace
+:- '$hide'('$abort'/0).			% same after an abort
 
 '$toplevel' :-
 	'$runtoplevel',
@@ -380,7 +400,7 @@ initialise_prolog :-
 %	crashing in a loop?
 
 '$runtoplevel' :-
-	'$option'(toplevel, TopLevelAtom, TopLevelAtom), 
+	'$option'(toplevel, TopLevelAtom, TopLevelAtom),
 	catch(term_to_atom(TopLevel, TopLevelAtom), E,
 	      (print_message(error, E),
 	       halt(1))),
@@ -390,7 +410,7 @@ initialise_prolog :-
 %	Toplevel called when invoked with -c option.
 
 '$compile' :-
-	'$run_at_initialization',
+	'$run_initialization',
 	'$load_system_init_file',
 	'$set_file_search_paths',
 	catch('$compile_wic', E, (print_message(error, E), halt(1))).
@@ -401,10 +421,10 @@ initialise_prolog :-
 		*********************************/
 
 prolog :-
-	flag('$tracing', _, off), 
-	flag('$break_level', BreakLev, BreakLev), 
-	repeat, 
-	    (   '$module'(TypeIn, TypeIn), 
+	flag('$tracing', _, off),
+	flag('$break_level', BreakLev, BreakLev),
+	repeat,
+	    (   '$module'(TypeIn, TypeIn),
 		(   stream_property(user_input, tty(true))
 		->  '$system_prompt'(TypeIn, BreakLev, Prompt),
 		    prompt(Old, '|    ')
@@ -433,9 +453,12 @@ read_query(Prompt, Goal, Bindings) :-
 	       ->  fail
 	       ;   throw(E)
 	       ))),
-	atom_concat(Line, '.', CompleteLine),
 	(   current_predicate(_, user:rl_add_history(_))
-	->  call(user:rl_add_history(CompleteLine))
+	->  format(atom(CompleteLine), '~W~W',
+		   [ Line, [partial(true)],
+		     '.', [partial(true)]
+		   ]),
+	    call(user:rl_add_history(CompleteLine))
 	;   true
 	),
 	catch(atom_to_term(Line, Goal, Bindings), E,
@@ -445,8 +468,8 @@ read_query(Prompt, Goal, Bindings) :-
 	'$save_history'(Line).
 read_query(Prompt, Goal, Bindings) :-
 	seeing(Old), see(user_input),
-	(   read_history(h, '!h', 
-			 [trace, end_of_file], 
+	(   read_history(h, '!h',
+			 [trace, end_of_file],
 			 Prompt, Goal, Bindings)
 	->  see(Old)
 	;   see(Old),
@@ -464,7 +487,7 @@ clean_history_prompt_chars([], []).
 clean_history_prompt_chars(['%', !|T], T) :- !.
 clean_history_prompt_chars([H|T0], [H|T]) :-
 	clean_history_prompt_chars(T0, T).
- 
+
 delete_leading_blanks([' '|T0], T) :- !,
 	delete_leading_blanks(T0, T).
 delete_leading_blanks(L, L).
@@ -474,13 +497,40 @@ set_default_history :-
 	(   (   current_prolog_flag(readline, true)
 	    ;	current_prolog_flag(emacs_inferior_process, true)
 	    )
-	->  set_prolog_flag(history, 0)
-	;   set_prolog_flag(history, 25)
+	->  create_prolog_flag(history, 0, [])
+	;   create_prolog_flag(history, 25, [])
 	).
 
 :- initialization set_default_history.
 
 
+		 /*******************************
+		 *	  TOPLEVEL DEBUG	*
+		 *******************************/
+
+save_debug :-
+	(   tracing,
+	    notrace
+	->  Tracing = true
+	;   Tracing = false
+	),
+	current_prolog_flag(debug, Debugging),
+	set_prolog_flag(debug, false),
+	create_prolog_flag(query_debug_settings,
+			   debug(Debugging, Tracing), []).
+
+restore_debug :-
+	current_prolog_flag(query_debug_settings, debug(Debugging, Tracing)),
+	set_prolog_flag(debug, Debugging),
+	(   Tracing == true
+	->  trace
+	;   true
+	).
+
+:- initialization
+	create_prolog_flag(query_debug_settings, debug(false, false), []).
+
+
 		/********************************
 		*            PROMPTING		*
 		********************************/
@@ -506,14 +556,15 @@ set_default_history :-
 	->   '$substitute'("%l", ["[", BrekLev, "] "], P1, P2)
 	;    '$substitute'("%l", [], P1, P2)
 	),
-	(    tracing
+	current_prolog_flag(query_debug_settings, debug(Debugging, Tracing)),
+	(    Tracing == true
 	->   '$substitute'("%d", ["[trace] "], P2, P3)
-	;    current_prolog_flag(debug, true)
+	;    Debugging == true
 	->   '$substitute'("%d", ["[debug] "], P2, P3)
 	;    '$substitute'("%d", [], P2, P3)
 	),
 	atom_chars(Prompt, P3).
-	
+
 '$substitute'(From, T, Old, New) :-
 	phrase(subst_chars(T), T0),
 	'$append'(Pre, S0, Old),
@@ -521,7 +572,7 @@ set_default_history :-
 	'$append'(Pre, T0, S1),
 	'$append'(S1, Post, New), !.
 '$substitute'(_, _, Old, Old).
-	
+
 subst_chars([]) -->
 	[].
 subst_chars([H|T]) -->
@@ -546,48 +597,44 @@ subst_chars([H|T]) -->
 '$execute'(end_of_file, _) :- !,
 	print_message(query, query(eof)).
 '$execute'(Goal, Bindings) :-
-	'$module'(TypeIn, TypeIn), 
-	expand_goal(Goal, Expanded),
-	TypeIn:'$dwim_correct_goal'(Expanded, Bindings, Corrected), !, 
-	'$execute_goal'(Corrected, Bindings).
+	'$module'(TypeIn, TypeIn),
+	'$dwim_correct_goal'(TypeIn:Goal, Bindings, Corrected), !,
+	setup_call_cleanup('$set_source_module'(M0, TypeIn),
+			   expand_goal(Corrected, Expanded),
+			   '$set_source_module'(_, M0)),
+	print_message(silent, toplevel_goal(Expanded, Bindings)),
+	'$execute_goal2'(Expanded, Bindings).
 '$execute'(_, _) :-
-	notrace, 
+	notrace,
 	print_message(query, query(no)),
 	fail.
 
-'$execute_goal'(trace, []) :-
-	trace, 
-	print_message(query, query(yes)), !,
-	fail.
-'$execute_goal'(Goal, Bindings) :-
-	'$module'(TypeIn, TypeIn),
-	print_message(silent, toplevel_goal(TypeIn:Goal, Bindings)),
-	'$execute_goal2'(TypeIn:Goal, Bindings).
-
 '$execute_goal2'(Goal, Bindings) :-
+	restore_debug,
 	Goal,
-	flush_output(user_output),
 	deterministic(Det),
+	(   save_debug
+	;   restore_debug, fail
+	),
+	flush_output(user_output),
 	call_expand_answer(Bindings, NewBindings),
-	(    write_bindings(NewBindings, Det)
-	->   !,
-	     notrace,
-	     fail
+	(    \+ \+ write_bindings(NewBindings, Det)
+	->   !, fail
 	).
 '$execute_goal2'(_, _) :-
-	notrace, 
+	save_debug,
 	print_message(query, query(no)),
 	fail.
 
 %%	write_bindings(+Bindings, +Deterministic)
-%	
+%
 %	Write   bindings   resulting   from   a     query.    The   flag
 %	prompt_alternatives_on determines whether the   user is prompted
 %	for alternatives. =groundness= gives   the  classical behaviour,
 %	=determinism= is considered more adequate and informative.
 
 write_bindings(Bindings, Det) :-
-	'$attributed'(Bindings), !,
+	\+ term_attvars(Bindings, []), !,
 	copy_term(Bindings, Bindings1, Residuals0),
 	'$module'(TypeIn, TypeIn),
 	omit_qualifiers(Residuals0, TypeIn, Residuals),
@@ -646,6 +693,8 @@ omit_qualifier(M:G0, _, M:G) :-
 omit_qualifier(G0, TypeIn, G) :-
 	omit_meta_qualifiers(G0, TypeIn, G).
 
+omit_meta_qualifiers(V, _, V) :-
+	var(V), !.
 omit_meta_qualifiers((QA,QB), TypeIn, (A,B)) :- !,
 	omit_qualifier(QA, TypeIn, A),
 	omit_qualifier(QB, TypeIn, B).
@@ -659,7 +708,7 @@ omit_meta_qualifiers(G, _, G).
 
 
 %%	bind_vars(+Bindings)
-%	
+%
 %	Bind variables to '$VAR'(Name), so they are printed by the names
 %	used in the query. Note that by   binding  in the reverse order,
 %	variables bound to one another come out in the natural order.
@@ -673,7 +722,7 @@ bind_vars([Name=Var|T]) :-
 	).
 
 %%	filter_bindings(+Bindings0, -Bindings)
-%	
+%
 %	Remove bindings that must not be printed.
 
 filter_bindings([], []).
@@ -709,6 +758,7 @@ answer_respons(Char, redo) :-
 answer_respons(Char, redo) :-
 	memberchk(Char, "tT"), !,
 	trace,
+	save_debug,
 	print_message(query, if_tty('; [trace]')).
 answer_respons(Char, continue) :-
 	memberchk(Char, "ca\n\ryY."), !,
@@ -726,12 +776,12 @@ answer_respons(Char, again) :-
 	print_message(query, no_action(Char)).
 
 print_predicate(0'w, [write], [ quoted(true),
-				priority(699)
+				spacing(next_argument)
 			      ]).
 print_predicate(0'p, [print], [ quoted(true),
 				portray(true),
 				max_depth(10),
-				priority(699)
+				spacing(next_argument)
 			      ]).
 
 
