@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemak@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2010, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,8 @@
 #ifndef _FLI_H_INCLUDED
 #define _FLI_H_INCLUDED
 
-#ifndef __SWI_PROLOG__			/* use this to switch on Prolog dialect */
-#define __SWI_PROLOG__			/* normally defined by the plld compiler driver */
+#ifndef __SWI_PROLOG__	/* use this to switch on Prolog dialect */
+#define __SWI_PROLOG__	/* normally defined by the swipl-ld compiler driver */
 #endif
 
 #if defined(_MSC_VER) && !defined(__WINDOWS__)
@@ -51,19 +51,10 @@ typedef unsigned long uintptr_t;
 extern "C" {
 #endif
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-This interface has been changed considerable  between version 2.1.14 and
-2.5.0. You may get most old code to work using
-
-	#define PL_OLD_INTERFACE
-
-before loading this file.  See end of this file.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 50659
+#define PLVERSION 51002
 #endif
 
 		 /*******************************
@@ -116,6 +107,17 @@ duplicated this stuff.
 #endif /*_PL_EXPORT_DONE*/
 
 
+		 /*******************************
+		 *	  GCC ATTRIBUTES	*
+		 *******************************/
+
+#if __GNUC__ >= 4
+#define WUNUSED __attribute__((warn_unused_result))
+#else
+#define WUNUSED
+#endif
+
+
 		 /*******************************
 		 *	       TYPES		*
 		 *******************************/
@@ -127,6 +129,7 @@ typedef Record		record_t;	/* handle to a recorded term */
 typedef struct PL_local_data *PL_engine_t; /* handle to a engine */
 #else
 typedef	uintptr_t	atom_t;		/* Prolog atom */
+typedef uintptr_t	functor_t;	/* Name/arity pair */
 typedef void *		module_t;	/* Prolog module */
 typedef void *		predicate_t;	/* Prolog procedure */
 typedef void *		record_t;	/* Prolog recorded term */
@@ -136,7 +139,6 @@ typedef uintptr_t	PL_fid_t;	/* opaque foreign context handle */
 typedef void *		control_t;	/* non-deterministic control arg */
 typedef void *		PL_engine_t;	/* opaque engine handle */
 #endif
-typedef uintptr_t	functor_t;	/* Name/arity pair */
 typedef uintptr_t	PL_atomic_t;	/* same a word */
 typedef uintptr_t	foreign_t;	/* return type of foreign functions */
 typedef wchar_t	        pl_wchar_t;	/* Prolog wide character */
@@ -188,7 +190,7 @@ typedef union
 					/* PlArg::PlArg(text, type) */
 #define PL_CODE_LIST	 (14)		/* [ascii...] */
 #define PL_CHAR_LIST	 (15)		/* [h,e,l,l,o] */
-#define PL_BOOL		 (16)		/* PL_set_feature() */
+#define PL_BOOL		 (16)		/* PL_set_prolog_flag() */
 #define PL_FUNCTOR_CHARS (17)		/* PL_unify_term() */
 #define _PL_PREDICATE_INDICATOR (18)	/* predicate_t (Procedure) */
 #define PL_SHORT	 (19)		/* short */
@@ -209,6 +211,20 @@ typedef union
 #define PL_MBCODES	 (35)		/* const char * */
 #define PL_MBSTRING	 (36)		/* const char * */
 #define PL_INTPTR	 (37)		/* intptr_t */
+#define PL_CHAR		 (38)		/* int */
+#define PL_CODE		 (39)		/* int */
+#define PL_BYTE		 (40)		/* int */
+					/* PL_skip_list() */
+#define PL_PARTIAL_LIST	 (41)		/* a partial list */
+#define PL_CYCLIC_TERM	 (42)		/* a cyclic list/term */
+#define PL_NOT_A_LIST	 (43)		/* Object is not a list */
+
+/* Or'ed flags for PL_set_prolog_flag() */
+/* MUST fit in a short int! */
+#define FF_READONLY	 0x1000		/* Read-only prolog flag */
+#define FF_KEEP		 0x2000		/* keep prolog flag if already set */
+#define FF_NOCREATE	 0x4000		/* Fail if flag is non-existent */
+#define FF_MASK		 0xf000
 
 
 		/********************************
@@ -226,11 +242,13 @@ typedef union
 /*  Note 1: Non-deterministic foreign functions may also use the deterministic
     return methods PL_succeed and PL_fail.
 
-    Note 2: The argument to PL_retry is a 30 bits signed integer (intptr_t).
+    Note 2: The argument to PL_retry is a sizeof(ptr)-2 bits signed
+    integer (use type intptr_t).
 */
 
 #define PL_FIRST_CALL		(0)
-#define PL_CUTTED		(1)
+#define PL_CUTTED		(1)	/* deprecated */
+#define PL_PRUNED		(1)
 #define PL_REDO			(2)
 
 #define PL_retry(n)		return _PL_retry(n)
@@ -239,7 +257,7 @@ typedef union
 PL_EXPORT(foreign_t)	_PL_retry(intptr_t);
 PL_EXPORT(foreign_t)	_PL_retry_address(void *);
 PL_EXPORT(int)	 	PL_foreign_control(control_t);
-PL_EXPORT(intptr_t)	 	PL_foreign_context(control_t);
+PL_EXPORT(intptr_t)	PL_foreign_context(control_t);
 PL_EXPORT(void *)	PL_foreign_context_address(control_t);
 
 
@@ -320,7 +338,7 @@ PL_EXPORT(int)		PL_predicate_info(predicate_t pred,
 			/* Call-back */
 PL_EXPORT(qid_t)	PL_open_query(module_t m, int flags,
 				      predicate_t pred, term_t t0);
-PL_EXPORT(int)		PL_next_solution(qid_t qid);
+PL_EXPORT(int)		PL_next_solution(qid_t qid) WUNUSED;
 PL_EXPORT(void)		PL_close_query(qid_t qid);
 PL_EXPORT(void)		PL_cut_query(qid_t qid);
 
@@ -332,6 +350,7 @@ PL_EXPORT(int)		PL_call_predicate(module_t m, int debug,
 PL_EXPORT(term_t)	PL_exception(qid_t qid);
 PL_EXPORT(int)		PL_raise_exception(term_t exception);
 PL_EXPORT(int)		PL_throw(term_t exception);
+PL_EXPORT(void)		PL_clear_exception(void);
 
 
 		 /*******************************
@@ -360,35 +379,36 @@ PL_EXPORT(atom_t)	PL_functor_name(functor_t f);
 PL_EXPORT(int)		PL_functor_arity(functor_t f);
 
 			/* Get C-values from Prolog terms */
-PL_EXPORT(int)		PL_get_atom(term_t t, atom_t *a);
-PL_EXPORT(int)		PL_get_bool(term_t t, int *value);
-PL_EXPORT(int)		PL_get_atom_chars(term_t t, char **a);
+PL_EXPORT(int)		PL_get_atom(term_t t, atom_t *a) WUNUSED;
+PL_EXPORT(int)		PL_get_bool(term_t t, int *value) WUNUSED;
+PL_EXPORT(int)		PL_get_atom_chars(term_t t, char **a) WUNUSED;
 #define PL_get_string_chars(t, s, l) PL_get_string(t,s,l)
-					/* PL_get_string() is depricated */
-PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len);
-PL_EXPORT(int)		PL_get_chars(term_t t, char **s, unsigned int flags);
+					/* PL_get_string() is deprecated */
+PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len); WUNUSED
+PL_EXPORT(int)		PL_get_chars(term_t t, char **s, unsigned int flags) WUNUSED;
 PL_EXPORT(int)		PL_get_list_chars(term_t l, char **s,
-					  unsigned int flags);
-PL_EXPORT(int)		PL_get_atom_nchars(term_t t, size_t *len, char **a);
+					  unsigned int flags) WUNUSED;
+PL_EXPORT(int)		PL_get_atom_nchars(term_t t, size_t *len, char **a) WUNUSED;
 PL_EXPORT(int)		PL_get_list_nchars(term_t l,
 					   size_t *len, char **s,
-					   unsigned int flags);
+					   unsigned int flags) WUNUSED;
 PL_EXPORT(int)		PL_get_nchars(term_t t,
 				      size_t *len, char **s,
-				      unsigned int flags);
-PL_EXPORT(int)		PL_get_integer(term_t t, int *i);
-PL_EXPORT(int)		PL_get_long(term_t t, long *i);
-PL_EXPORT(int)		PL_get_pointer(term_t t, void **ptr);
-PL_EXPORT(int)		PL_get_float(term_t t, double *f);
-PL_EXPORT(int)		PL_get_functor(term_t t, functor_t *f);
-PL_EXPORT(int)		PL_get_name_arity(term_t t, atom_t *name, int *arity);
-PL_EXPORT(int)		PL_get_module(term_t t, module_t *module);
-PL_EXPORT(int)		PL_get_arg(int index, term_t t, term_t a);
-PL_EXPORT(int)		PL_get_list(term_t l, term_t h, term_t t);
-PL_EXPORT(int)		PL_get_head(term_t l, term_t h);
-PL_EXPORT(int)		PL_get_tail(term_t l, term_t t);
-PL_EXPORT(int)		PL_get_nil(term_t l);
-PL_EXPORT(int)		PL_get_term_value(term_t t, term_value_t *v);
+				      unsigned int flags) WUNUSED;
+PL_EXPORT(int)		PL_get_integer(term_t t, int *i) WUNUSED;
+PL_EXPORT(int)		PL_get_long(term_t t, long *i) WUNUSED;
+PL_EXPORT(int)		PL_get_intptr(term_t t, intptr_t *i) WUNUSED;
+PL_EXPORT(int)		PL_get_pointer(term_t t, void **ptr) WUNUSED;
+PL_EXPORT(int)		PL_get_float(term_t t, double *f) WUNUSED;
+PL_EXPORT(int)		PL_get_functor(term_t t, functor_t *f) WUNUSED;
+PL_EXPORT(int)		PL_get_name_arity(term_t t, atom_t *name, int *arity) WUNUSED;
+PL_EXPORT(int)		PL_get_module(term_t t, module_t *module) WUNUSED;
+PL_EXPORT(int)		PL_get_arg(int index, term_t t, term_t a) WUNUSED;
+PL_EXPORT(int)		PL_get_list(term_t l, term_t h, term_t t) WUNUSED;
+PL_EXPORT(int)		PL_get_head(term_t l, term_t h) WUNUSED;
+PL_EXPORT(int)		PL_get_tail(term_t l, term_t t) WUNUSED;
+PL_EXPORT(int)		PL_get_nil(term_t l) WUNUSED;
+PL_EXPORT(int)		PL_get_term_value(term_t t, term_value_t *v) WUNUSED;
 PL_EXPORT(char *)	PL_quote(int chr, const char *data);
 
 			/* Verify types */
@@ -409,52 +429,59 @@ PL_EXPORT(int)		PL_is_number(term_t t);
 PL_EXPORT(int)		PL_is_acyclic(term_t t);
 
 			/* Assign to term-references */
-PL_EXPORT(void)		PL_put_variable(term_t t);
-PL_EXPORT(void)		PL_put_atom(term_t t, atom_t a);
-PL_EXPORT(void)		PL_put_atom_chars(term_t t, const char *chars);
-PL_EXPORT(void)		PL_put_string_chars(term_t t, const char *chars);
-PL_EXPORT(void)		PL_put_list_chars(term_t t, const char *chars);
-PL_EXPORT(void)		PL_put_list_codes(term_t t, const char *chars);
-PL_EXPORT(void)		PL_put_atom_nchars(term_t t, size_t l, const char *chars);
-PL_EXPORT(void)		PL_put_string_nchars(term_t t, size_t len, const char *chars);
-PL_EXPORT(void)		PL_put_list_nchars(term_t t, size_t l, const char *chars);
-PL_EXPORT(void)		PL_put_list_ncodes(term_t t, size_t l, const char *chars);
-PL_EXPORT(void)		PL_put_integer(term_t t, long i);
-PL_EXPORT(void)		PL_put_pointer(term_t t, void *ptr);
-PL_EXPORT(void)		PL_put_float(term_t t, double f);
-PL_EXPORT(void)		PL_put_functor(term_t t, functor_t functor);
-PL_EXPORT(void)		PL_put_list(term_t l);
+PL_EXPORT(int)		PL_put_variable(term_t t);
+PL_EXPORT(int)		PL_put_atom(term_t t, atom_t a);
+PL_EXPORT(int)		PL_put_atom_chars(term_t t, const char *chars);
+PL_EXPORT(int)		PL_put_string_chars(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_list_chars(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_list_codes(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_atom_nchars(term_t t, size_t l, const char *chars);
+PL_EXPORT(int)		PL_put_string_nchars(term_t t, size_t len, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_list_nchars(term_t t, size_t l, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_list_ncodes(term_t t, size_t l, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_put_integer(term_t t, long i) WUNUSED;
+PL_EXPORT(int)		PL_put_pointer(term_t t, void *ptr) WUNUSED;
+PL_EXPORT(int)		PL_put_float(term_t t, double f) WUNUSED;
+PL_EXPORT(int)		PL_put_functor(term_t t, functor_t functor) WUNUSED;
+PL_EXPORT(int)		PL_put_list(term_t l) WUNUSED;
 PL_EXPORT(void)		PL_put_nil(term_t l);
-PL_EXPORT(void)		PL_put_term(term_t t1, term_t t2);
+PL_EXPORT(int)		PL_put_term(term_t t1, term_t t2);
 
 			/* construct a functor or list-cell */
-PL_EXPORT(void)		PL_cons_functor(term_t h, functor_t f, ...);
-PL_EXPORT(void)		PL_cons_functor_v(term_t h, functor_t fd, term_t a0);
-PL_EXPORT(void)		PL_cons_list(term_t l, term_t h, term_t t);
+PL_EXPORT(int)		PL_cons_functor(term_t h, functor_t f, ...) WUNUSED;
+PL_EXPORT(int)		PL_cons_functor_v(term_t h, functor_t fd, term_t a0) WUNUSED;
+PL_EXPORT(int)		PL_cons_list(term_t l, term_t h, term_t t) WUNUSED;
 
 			/* Unify term-references */
-PL_EXPORT(int)		PL_unify(term_t t1, term_t t2);
-PL_EXPORT(int)		PL_unify_atom(term_t t, atom_t a);
-PL_EXPORT(int)		PL_unify_atom_chars(term_t t, const char *chars);
-PL_EXPORT(int)		PL_unify_list_chars(term_t t, const char *chars);
-PL_EXPORT(int)		PL_unify_list_codes(term_t t, const char *chars);
-PL_EXPORT(int)		PL_unify_string_chars(term_t t, const char *chars);
-PL_EXPORT(int)		PL_unify_atom_nchars(term_t t, size_t l, const char *s);
-PL_EXPORT(int)		PL_unify_list_ncodes(term_t t, size_t l, const char *s);
-PL_EXPORT(int)		PL_unify_list_nchars(term_t t, size_t l, const char *s);
+PL_EXPORT(int)		PL_unify(term_t t1, term_t t2) WUNUSED;
+PL_EXPORT(int)		PL_unify_atom(term_t t, atom_t a) WUNUSED;
+PL_EXPORT(int)		PL_unify_atom_chars(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_unify_list_chars(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_unify_list_codes(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_unify_string_chars(term_t t, const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_unify_atom_nchars(term_t t, size_t l, const char *s) WUNUSED;
+PL_EXPORT(int)		PL_unify_list_ncodes(term_t t, size_t l, const char *s) WUNUSED;
+PL_EXPORT(int)		PL_unify_list_nchars(term_t t, size_t l, const char *s) WUNUSED;
 PL_EXPORT(int)		PL_unify_string_nchars(term_t t,
 					       size_t len,
-					       const char *chars);
-PL_EXPORT(int)		PL_unify_integer(term_t t, intptr_t n);
-PL_EXPORT(int)		PL_unify_float(term_t t, double f);
-PL_EXPORT(int)		PL_unify_pointer(term_t t, void *ptr);
-PL_EXPORT(int)		PL_unify_functor(term_t t, functor_t f);
-PL_EXPORT(int)		PL_unify_list(term_t l, term_t h, term_t t);
-PL_EXPORT(int)		PL_unify_nil(term_t l);
-PL_EXPORT(int)		PL_unify_arg(int index, term_t t, term_t a);
-PL_EXPORT(int)		PL_unify_term(term_t t, ...);
+					       const char *chars) WUNUSED;
+PL_EXPORT(int)		PL_unify_bool(term_t t, int n) WUNUSED;
+PL_EXPORT(int)		PL_unify_integer(term_t t, intptr_t n) WUNUSED;
+PL_EXPORT(int)		PL_unify_float(term_t t, double f) WUNUSED;
+PL_EXPORT(int)		PL_unify_pointer(term_t t, void *ptr) WUNUSED;
+PL_EXPORT(int)		PL_unify_functor(term_t t, functor_t f) WUNUSED;
+PL_EXPORT(int)		PL_unify_list(term_t l, term_t h, term_t t) WUNUSED;
+PL_EXPORT(int)		PL_unify_nil(term_t l) WUNUSED;
+PL_EXPORT(int)		PL_unify_arg(int index, term_t t, term_t a) WUNUSED;
+PL_EXPORT(int)		PL_unify_term(term_t t, ...) WUNUSED;
 PL_EXPORT(int)		PL_unify_chars(term_t t, int flags,
-				       size_t len, const char *s);
+				       size_t len, const char *s) WUNUSED;
+
+		 /*******************************
+		 *	       LISTS		*
+		 *******************************/
+
+PL_EXPORT(int)		PL_skip_list(term_t list, term_t tail, size_t *len);
 
 
 		 /*******************************
@@ -462,13 +489,13 @@ PL_EXPORT(int)		PL_unify_chars(term_t t, int flags,
 		 *******************************/
 
 PL_EXPORT(int)		PL_unify_wchars(term_t t, int type,
-					size_t len, const pl_wchar_t *s);
+					size_t len, const pl_wchar_t *s) WUNUSED;
 PL_EXPORT(int)		PL_unify_wchars_diff(term_t t, term_t tail, int type,
-					size_t len, const pl_wchar_t *s);
+					size_t len, const pl_wchar_t *s) WUNUSED;
 PL_EXPORT(int)		PL_get_wchars(term_t l,
 				      size_t *length, pl_wchar_t **s,
-				      unsigned flags);
-PL_EXPORT(size_t)	PL_utf8_strlen(const char *s, size_t len);
+				      unsigned flags) WUNUSED;
+PL_EXPORT(size_t)	PL_utf8_strlen(const char *s, size_t len) WUNUSED;
 
 
 		 /*******************************
@@ -476,9 +503,9 @@ PL_EXPORT(size_t)	PL_utf8_strlen(const char *s, size_t len);
 		 *******************************/
 
 
-PL_EXPORT(int)		PL_get_int64(term_t t, int64_t *i);
-PL_EXPORT(int)		PL_unify_int64(term_t t, int64_t value);
-PL_EXPORT(void)		PL_put_int64(term_t t, int64_t i);
+PL_EXPORT(int)		PL_get_int64(term_t t, int64_t *i) WUNUSED;
+PL_EXPORT(int)		PL_unify_int64(term_t t, int64_t value) WUNUSED;
+PL_EXPORT(int)		PL_put_int64(term_t t, int64_t i) WUNUSED;
 
 
 		 /*******************************
@@ -552,10 +579,10 @@ PL_EXPORT(int)		PL_unregister_blob_type(PL_blob_t *type);
 		 *	       GMP		*
 		 *******************************/
 
-PL_EXPORT(int)	PL_get_mpz(term_t t, mpz_t mpz);
-PL_EXPORT(int)	PL_get_mpq(term_t t,  mpq_t mpq);
-PL_EXPORT(int)	PL_unify_mpz(term_t t, mpz_t mpz);
-PL_EXPORT(int)	PL_unify_mpq(term_t t, mpq_t mpq);
+PL_EXPORT(int)	PL_get_mpz(term_t t, mpz_t mpz) WUNUSED;
+PL_EXPORT(int)	PL_get_mpq(term_t t,  mpq_t mpq) WUNUSED;
+PL_EXPORT(int)	PL_unify_mpz(term_t t, mpz_t mpz) WUNUSED;
+PL_EXPORT(int)	PL_unify_mpq(term_t t, mpq_t mpq) WUNUSED;
 
 #endif /*__GNU_MP__*/
 
@@ -578,19 +605,35 @@ PL_EXPORT(const char *) PL_cwd(void);
 
 
 		 /*******************************
-		 *   QUINTUS WRAPPER SUPPORT	*
+		 *    QUINTUS/SICSTUS WRAPPER	*
 		 *******************************/
 
-PL_EXPORT(int)		PL_cvt_i_integer(term_t p, long *c);
+PL_EXPORT(int)		PL_cvt_i_int(term_t p, int *c);
+PL_EXPORT(int)		PL_cvt_i_long(term_t p, long *c);
+PL_EXPORT(int)		PL_cvt_i_size_t(term_t p, size_t *c);
 PL_EXPORT(int)		PL_cvt_i_float(term_t p, double *c);
 PL_EXPORT(int)		PL_cvt_i_single(term_t p, float *c);
 PL_EXPORT(int)		PL_cvt_i_string(term_t p, char **c);
+PL_EXPORT(int)		PL_cvt_i_codes(term_t p, char **c);
 PL_EXPORT(int)		PL_cvt_i_atom(term_t p, atom_t *c);
-PL_EXPORT(int)		PL_cvt_o_integer(intptr_t c, term_t p);
+PL_EXPORT(int)		PL_cvt_i_address(term_t p, void *c);
+PL_EXPORT(int)		PL_cvt_o_int64(int64_t c, term_t p);
 PL_EXPORT(int)		PL_cvt_o_float(double c, term_t p);
 PL_EXPORT(int)		PL_cvt_o_single(float c, term_t p);
 PL_EXPORT(int)		PL_cvt_o_string(const char *c, term_t p);
+PL_EXPORT(int)		PL_cvt_o_codes(const char *c, term_t p);
 PL_EXPORT(int)		PL_cvt_o_atom(atom_t c, term_t p);
+PL_EXPORT(int)		PL_cvt_o_address(void *address, term_t p);
+PL_EXPORT(term_t)	PL_new_nil_ref(void);
+
+/* set/get encoding for PL_cvt_*_string() functions.  The default
+   is UTF-8 (REP_UTF8)
+*/
+
+PL_EXPORT(int)		PL_cvt_encoding(void);
+PL_EXPORT(int)		PL_cvt_set_encoding(int enc);
+PL_EXPORT(void)		SP_set_state(int state);
+PL_EXPORT(int)		SP_get_state(void);
 
 
 		 /*******************************
@@ -612,7 +655,7 @@ PL_EXPORT(void)		PL_fatal_error(const char *fmt, ...);
 		 *******************************/
 
 PL_EXPORT(record_t)	PL_record(term_t term);
-PL_EXPORT(void)		PL_recorded(record_t record, term_t term);
+PL_EXPORT(int)		PL_recorded(record_t record, term_t term);
 PL_EXPORT(void)		PL_erase(record_t record);
 
 PL_EXPORT(char *)	PL_record_external(term_t t, size_t *size);
@@ -620,10 +663,12 @@ PL_EXPORT(int)		PL_recorded_external(const char *rec, term_t term);
 PL_EXPORT(int)		PL_erase_external(char *rec);
 
 		 /*******************************
-		 *	      FEATURES		*
+		 *	   PROLOG FLAGS		*
 		 *******************************/
 
-PL_EXPORT(int)		PL_set_feature(const char *name, int type, ...);
+#define PL_set_feature  PL_set_prolog_flag /* compatibility */
+PL_EXPORT(int)		PL_set_prolog_flag(const char *name, int type, ...);
+
 
 		 /*******************************
 		 *	INTERNAL FUNCTIONS	*
@@ -632,7 +677,6 @@ PL_EXPORT(int)		PL_set_feature(const char *name, int type, ...);
 PL_EXPORT(PL_atomic_t)	_PL_get_atomic(term_t t);
 PL_EXPORT(void)		_PL_put_atomic(term_t t, PL_atomic_t a);
 PL_EXPORT(int)		_PL_unify_atomic(term_t t, PL_atomic_t a);
-PL_EXPORT(void)		_PL_copy_atomic(term_t t, PL_atomic_t a);
 PL_EXPORT(void)		_PL_get_arg(int index, term_t t, term_t a);
 
 
@@ -648,7 +692,8 @@ PL_EXPORT(void)		_PL_get_arg(int index, term_t t, term_t a);
 #define CVT_VARIABLE	0x0020
 #define CVT_NUMBER	(CVT_INTEGER|CVT_FLOAT)
 #define CVT_ATOMIC	(CVT_NUMBER|CVT_ATOM|CVT_STRING)
-#define CVT_WRITE	0x0040		/* as of version 3.2.10 */
+#define CVT_WRITE	0x0040
+#define CVT_WRITE_CANONICAL 0x0080
 #define CVT_ALL		(CVT_ATOMIC|CVT_LIST)
 #define CVT_MASK	0x00ff
 
@@ -657,6 +702,7 @@ PL_EXPORT(void)		_PL_get_arg(int index, term_t t, term_t a);
 #define BUF_MALLOC	0x0200
 
 #define CVT_EXCEPTION	0x10000		/* throw exception on error */
+#define CVT_VARNOFAIL	0x20000		/* return 2 if argument is unbound */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Output   representation   for   PL_get_chars()     and    friends.   The
@@ -683,7 +729,7 @@ UNICODE file functions.
 		 *******************************/
 
 					/* Make IOSTREAM known to Prolog */
-PL_EXPORT(int)  	PL_open_stream(term_t t, IOSTREAM *s); /* compat */
+#define PL_open_stream  PL_unify_stream	/* compatibility */
 PL_EXPORT(int)  	PL_unify_stream(term_t t, IOSTREAM *s);
 PL_EXPORT(int)  	PL_get_stream_handle(term_t t, IOSTREAM **s);
 PL_EXPORT(int) 		PL_release_stream(IOSTREAM *s);
@@ -715,11 +761,12 @@ PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 	 PL_WRT_ATTVAR_DOTS | \
 	 PL_WRT_ATTVAR_WRITE | \
 	 PL_WRT_ATTVAR_PORTRAY)
+#define PL_WRT_BLOB_PORTRAY	0x400	/* Use portray to emit non-text blobs */
 
 PL_EXPORT(int) PL_write_term(IOSTREAM *s,
-			      term_t term,
-			      int precedence,
-			      int flags);
+			     term_t term,
+			     int precedence,
+			     int flags);
 
 					/* PL_ttymode() results */
 #define PL_NOTTY	0		/* -tty in effect */
@@ -776,7 +823,7 @@ PL_EXPORT(pl_wchar_t*)  PL_atom_generator_w(const pl_wchar_t *pref,
 					    pl_wchar_t *buffer,
 					    size_t buflen,
 					    int state);
-PL_EXPORT(void)		PL_clock_wait_ticks(intptr_t waited);
+PL_EXPORT(void)		PL_clock_wait_ticks(long waited);
 
 
 		 /*******************************
@@ -797,14 +844,12 @@ PL_EXPORT(void)		PL_free(void *mem);
 typedef int  (*PL_dispatch_hook_t)(int fd);
 typedef void (*PL_abort_hook_t)(void);
 typedef void (*PL_initialise_hook_t)(int argc, char **argv);
-typedef void (*PL_async_hook_t)(void);	/* Win32 only (O_ASYNC_HOOK) */
 typedef int  (*PL_agc_hook_t)(atom_t a);
 
 PL_EXPORT(PL_dispatch_hook_t) 	PL_dispatch_hook(PL_dispatch_hook_t);
 PL_EXPORT(void)	       		PL_abort_hook(PL_abort_hook_t);
 PL_EXPORT(void)	       		PL_initialise_hook(PL_initialise_hook_t);
 PL_EXPORT(int)		      	PL_abort_unhook(PL_abort_hook_t);
-PL_EXPORT(PL_async_hook_t)    	PL_async_hook(unsigned int, PL_async_hook_t);
 PL_EXPORT(PL_agc_hook_t)      	PL_agc_hook(PL_agc_hook_t);
 
 
@@ -813,6 +858,7 @@ PL_EXPORT(PL_agc_hook_t)      	PL_agc_hook(PL_agc_hook_t);
 		*********************************/
 
 #define PL_SIGSYNC	0x00010000	/* call handler synchronously */
+#define PL_SIGNOFRAME	0x00020000	/* Do not create a Prolog frame */
 
 PL_EXPORT(void) (*PL_signal(int sig, void (*func)(int)))(int);
 PL_EXPORT(void) PL_interrupt(int sig);
@@ -858,6 +904,7 @@ PL_EXPORT(void) PL_on_halt(void (*)(int, void *), void *);
 #define PL_QUERY_MAX_THREADS	11	/* maximum thread count */
 #define PL_QUERY_ENCODING	12	/* I/O encoding */
 #define PL_QUERY_USER_CPU	13	/* User CPU in milliseconds */
+#define PL_QUERY_HALTING	14	/* If TRUE, we are in PL_cleanup() */
 
 PL_EXPORT(intptr_t)	PL_query(int);	/* get information from Prolog */
 
@@ -880,13 +927,17 @@ typedef struct
 PL_EXPORT(int)	PL_thread_self(void);	/* Prolog thread id (-1 if none) */
 PL_EXPORT(int)  PL_unify_thread_id(term_t t, int i);
 PL_EXPORT(int)	PL_get_thread_id_ex(term_t t, int *idp);
+PL_EXPORT(int)	PL_get_thread_alias(int tid, atom_t *alias);	/* Locks alias */
 PL_EXPORT(int)	PL_thread_attach_engine(PL_thread_attr_t *attr);
 PL_EXPORT(int)	PL_thread_destroy_engine(void);
 PL_EXPORT(int)	PL_thread_at_exit(void (*function)(void *),
 				  void *closure,
 				  int global);
+PL_EXPORT(int)	PL_thread_raise(int tid, int sig);
 #if defined(_WINDOWS_)			/* <windows.h> is included */
 PL_EXPORT(int) PL_w32thread_raise(DWORD dwTid, int sig);
+PL_EXPORT(int) PL_wait_for_console_input(void *handle);
+PL_EXPORT(int) PL_w32_wrap_ansi_console(void);
 #endif
 
 		 /*******************************
@@ -949,88 +1000,19 @@ typedef struct
   } value;
 } xpceref_t;
 
-PL_EXPORT(int)	_PL_get_xpce_reference(term_t t, xpceref_t *ref);
-PL_EXPORT(int)  _PL_unify_xpce_reference(term_t t, xpceref_t *ref);
-PL_EXPORT(void) _PL_put_xpce_reference_i(term_t t, uintptr_t r);
-PL_EXPORT(void) _PL_put_xpce_reference_a(term_t t, atom_t name);
+PL_EXPORT(int) _PL_get_xpce_reference(term_t t, xpceref_t *ref);
+PL_EXPORT(int) _PL_unify_xpce_reference(term_t t, xpceref_t *ref);
+PL_EXPORT(int) _PL_put_xpce_reference_i(term_t t, uintptr_t r);
+PL_EXPORT(int) _PL_put_xpce_reference_a(term_t t, atom_t name);
 
-		 /*******************************
-		 *        COMPATIBILITY		*
-		 *******************************/
-
-#ifdef PL_OLD_INTERFACE
-
-typedef term_t term;
-typedef PL_atomic_t atomic;
-
-#ifndef _PL_INCLUDE_H
-					/* renamed functions */
-#define PL_is_var(t)		PL_is_variable(t)
-#define PL_is_int(t)		PL_is_integer(t)
-#define PL_is_term(t)		PL_is_compound(t)
-#define PL_type(t)		PL_term_type(t)
-#define PL_atom_value(a)	(char *)PL_atom_chars((atom_t)(a))
-#define PL_predicate(f, m)	PL_pred(f, m)
-
-					/* force undefined symbols */
-					/* if PL_OLD_INTERFACE isn't set */
-#define PL_strip_module(t, m)	_PL_strip_module(t, m)
-#define PL_atomic(t)		_PL_atomic(t)
-#define PL_float_value(t)	_PL_float_value(t)
-#define PL_integer_value(t)	_PL_integer_value(t)
-#define PL_string_value(t)	_PL_string_value(t)
-#define PL_functor(t)		_PL_functor(t)
-#define PL_arg(t, n)		_PL_arg(t, n)
-#define PL_new_term()		_PL_new_term()
-#define PL_new_integer(i)	_PL_new_integer(i)
-#define PL_new_float(f)		_PL_new_float(f)
-#define PL_new_string(s)	_PL_new_string(s)
-#define PL_new_var()		_PL_new_var()
-#define PL_term(a)		_PL_term(a)
-#define PL_unify_atomic(t, a)	_PL_unify_atomic(t, (PL_atomic_t) (a))
-
-typedef fid_t			bktrk_buf;
-#define PL_mark(b)		(*(b) = PL_open_foreign_frame())
-#define PL_bktrk(b)		PL_discard_foreign_frame(*(b))
-#endif /*_PL_INCLUDE_H*/
-
-		 /*******************************
-		 *	     ANALYSIS		*
-		 *******************************/
-
-PL_EXPORT(PL_atomic_t)	_PL_atomic(term_t t);
-PL_EXPORT(int64_t)	_PL_integer_value(PL_atomic_t t);
-PL_EXPORT(double)	_PL_float_value(PL_atomic_t t);
-PL_EXPORT(char *)	_PL_string_value(PL_atomic_t t);
-PL_EXPORT(char *)	_PL_list_string_value(term_t t);
-PL_EXPORT(functor_t)	_PL_functor(term_t t);
-PL_EXPORT(term_t)	_PL_arg(term_t t, int n);
 
 
 		 /*******************************
-		 *	     CONSTRUCT		*
+		 *         TRACE SUPPORT	*
 		 *******************************/
 
-PL_EXPORT(term_t)	_PL_new_term(void);
-PL_EXPORT(PL_atomic_t)	_PL_new_integer(intptr_t i);
-PL_EXPORT(PL_atomic_t)	_PL_new_float(double f);
-PL_EXPORT(PL_atomic_t)	_PL_new_string(const char *s);
-PL_EXPORT(PL_atomic_t)	_PL_new_var(void);
-PL_EXPORT(term_t)	_PL_term(PL_atomic_t a);
+PL_EXPORT(int) PL_walk_prolog_stack(void *ref, char* buf, size_t len, void** nextref);
 
-		 /*******************************
-		 *	       UNIFY		*
-		 *******************************/
-
-PL_EXPORT(int)		_PL_unify_atomic(term_t t, PL_atomic_t a);
-
-		 /*******************************
-		 *	       MODULES		*
-		 *******************************/
-
-PL_EXPORT(term_t)	_PL_strip_module(term_t t, module_t *m);
-
-#endif /*PL_OLD_INTERFACE*/
 
 #ifdef __cplusplus
 }
