@@ -5,6 +5,42 @@
     Purpose: Provide index for autoload
 */
 
+index((r_open), 0, r_session, 'R').
+index((r_open), 1, r_session, 'R').
+index((r_close), 0, r_session, 'R').
+index((r_close), 1, r_session, 'R').
+index((r_in), 1, r_session, 'R').
+index((r_in), 2, r_session, 'R').
+index((r_push), 1, r_session, 'R').
+index((r_push), 2, r_session, 'R').
+index((r_out), 2, r_session, 'R').
+index((r_out), 3, r_session, 'R').
+index((r_err), 3, r_session, 'R').
+index((r_err), 4, r_session, 'R').
+index((r_print), 1, r_session, 'R').
+index((r_print), 2, r_session, 'R').
+index((r_lines_print), 1, r_session, 'R').
+index((r_lines_print), 2, r_session, 'R').
+index((r_lines_print), 3, r_session, 'R').
+index((r_lib), 1, r_session, 'R').
+index((r_lib), 2, r_session, 'R').
+index((r_flush), 0, r_session, 'R').
+index((r_flush), 1, r_session, 'R').
+index((r_flush_onto), 2, r_session, 'R').
+index((r_flush_onto), 3, r_session, 'R').
+index((current_r_session), 1, r_session, 'R').
+index((current_r_session), 3, r_session, 'R').
+index((default_r_session), 1, r_session, 'R').
+index((r_session_data), 3, r_session, 'R').
+index((r_streams_data), 3, r_session, 'R').
+index((r_history), 0, r_session, 'R').
+index((r_history), 1, r_session, 'R').
+index((r_history), 2, r_session, 'R').
+index((r_session_version), 1, r_session, 'R').
+index((r_bin), 1, r_session, 'R').
+index((r_bin_version), 1, r_session, 'R').
+index((r_bin_version), 2, r_session, 'R').
+index((r_verbosity), 1, r_session, 'R').
 index((foreach), 2, aggregate, aggregate).
 index((aggregate), 3, aggregate, aggregate).
 index((aggregate), 4, aggregate, aggregate).
@@ -18,7 +54,12 @@ index((include), 3, apply, apply).
 index((exclude), 3, apply, apply).
 index((partition), 4, apply, apply).
 index((partition), 5, apply, apply).
+index((maplist), 2, apply, apply).
+index((maplist), 3, apply, apply).
+index((maplist), 4, apply, apply).
+index((maplist), 5, apply, apply).
 index((empty_assoc), 1, assoc, assoc).
+index((is_assoc), 1, assoc, assoc).
 index((assoc_to_list), 2, assoc, assoc).
 index((assoc_to_keys), 2, assoc, assoc).
 index((assoc_to_values), 2, assoc, assoc).
@@ -32,16 +73,23 @@ index((max_assoc), 3, assoc, assoc).
 index((min_assoc), 3, assoc, assoc).
 index((ord_list_to_assoc), 2, assoc, assoc).
 index((put_assoc), 4, assoc, assoc).
+index((del_assoc), 4, assoc, assoc).
+index((del_min_assoc), 4, assoc, assoc).
+index((del_max_assoc), 4, assoc, assoc).
 index(('$arch'), 2, backward_compatibility, backcomp).
 index(('$version'), 1, backward_compatibility, backcomp).
 index(('$home'), 1, backward_compatibility, backcomp).
 index(('$argv'), 1, backward_compatibility, backcomp).
 index(('$strip_module'), 3, backward_compatibility, backcomp).
+index(('$declare_module'), 3, backward_compatibility, backcomp).
+index((at_initialization), 1, backward_compatibility, backcomp).
 index((displayq), 1, backward_compatibility, backcomp).
 index((displayq), 2, backward_compatibility, backcomp).
 index((sformat), 2, backward_compatibility, backcomp).
 index((sformat), 3, backward_compatibility, backcomp).
 index((concat), 3, backward_compatibility, backcomp).
+index((concat_atom), 2, backward_compatibility, backcomp).
+index((concat_atom), 3, backward_compatibility, backcomp).
 index((read_variables), 2, backward_compatibility, backcomp).
 index((read_variables), 3, backward_compatibility, backcomp).
 index((feature), 2, backward_compatibility, backcomp).
@@ -51,6 +99,8 @@ index((flush), 0, backward_compatibility, backcomp).
 index((write_ln), 1, backward_compatibility, backcomp).
 index((proper_list), 1, backward_compatibility, backcomp).
 index((free_variables), 2, backward_compatibility, backcomp).
+index((subsumes_chk), 2, backward_compatibility, backcomp).
+index((subsumes), 2, backward_compatibility, backcomp).
 index((hash_term), 2, backward_compatibility, backcomp).
 index((checklist), 2, backward_compatibility, backcomp).
 index((sublist), 3, backward_compatibility, backcomp).
@@ -62,6 +112,12 @@ index((current_mutex), 3, backward_compatibility, backcomp).
 index((message_queue_size), 2, backward_compatibility, backcomp).
 index((lock_predicate), 2, backward_compatibility, backcomp).
 index((unlock_predicate), 2, backward_compatibility, backcomp).
+index((current_module), 2, backward_compatibility, backcomp).
+index((export_list), 2, backward_compatibility, backcomp).
+index((setup_and_call_cleanup), 3, backward_compatibility, backcomp).
+index((setup_and_call_cleanup), 4, backward_compatibility, backcomp).
+index((merge), 3, backward_compatibility, backcomp).
+index((merge_set), 3, backward_compatibility, backcomp).
 index((base64), 2, base64, base64).
 index((base64), 3, base64, base64).
 index((please), 2, bim, bim).
@@ -90,8 +146,23 @@ index((listening), 3, broadcast, broadcast).
 index((broadcast), 1, broadcast, broadcast).
 index((broadcast_request), 1, broadcast, broadcast).
 index((cgi_get_form), 1, cgi, cgi).
+index((format_to_chars), 3, charsio, charsio).
+index((format_to_chars), 4, charsio, charsio).
+index((write_to_chars), 2, charsio, charsio).
+index((write_to_chars), 3, charsio, charsio).
+index((atom_to_chars), 2, charsio, charsio).
+index((atom_to_chars), 3, charsio, charsio).
+index((number_to_chars), 2, charsio, charsio).
+index((number_to_chars), 3, charsio, charsio).
+index((read_from_chars), 2, charsio, charsio).
+index((read_term_from_chars), 3, charsio, charsio).
+index((open_chars_stream), 2, charsio, charsio).
+index((with_output_to_chars), 2, charsio, charsio).
+index((with_output_to_chars), 3, charsio, charsio).
+index((with_output_to_chars), 4, charsio, charsio).
 index((check), 0, check, check).
 index((list_undefined), 0, check, check).
+index((list_undefined), 1, check, check).
 index((list_autoload), 0, check, check).
 index((list_redefined), 0, check, check).
 index((check_old_last), 0, checklast, checklast).
@@ -101,7 +172,14 @@ index((find_chr_constraint), 1, chr, chr).
 index((chr_trace), 0, chr, chr).
 index((chr_notrace), 0, chr, chr).
 index((chr_leash), 1, chr, chr).
+index((coinductive), 1, coinduction, coinduction).
 index((crypt), 2, crypt, crypt).
+index((csv), 3, csv, csv).
+index((csv), 4, csv, csv).
+index((csv_read_file), 2, csv, csv).
+index((csv_read_file), 3, csv, csv).
+index((csv_write_file), 2, csv, csv).
+index((csv_write_file), 3, csv, csv).
 index((is_alnum), 1, ctypes, ctypes).
 index((is_alpha), 1, ctypes, ctypes).
 index((is_ascii), 1, ctypes, ctypes).
@@ -128,12 +206,15 @@ index((to_lower), 2, ctypes, ctypes).
 index((to_upper), 2, ctypes, ctypes).
 index((date_time_value), 3, date, date).
 index((parse_time), 2, date, date).
+index((parse_time), 3, date, date).
+index((day_of_the_week), 2, date, date).
 index((debug), 3, prolog_debug, debug).
 index((debug), 1, prolog_debug, debug).
 index((nodebug), 1, prolog_debug, debug).
 index((debugging), 1, prolog_debug, debug).
 index((debugging), 2, prolog_debug, debug).
 index((list_debug_topics), 0, prolog_debug, debug).
+index((debug_message_context), 1, prolog_debug, debug).
 index((assertion), 1, prolog_debug, debug).
 index((expects_dialect), 1, prolog_dialect, dialect).
 index((exists_source), 1, prolog_dialect, dialect).
@@ -143,7 +224,6 @@ index((doc_server), 1, pldoc_http, doc_http).
 index((doc_server), 2, pldoc_http, doc_http).
 index((doc_browser), 0, pldoc_http, doc_http).
 index((doc_browser), 1, pldoc_http, doc_http).
-index((doc_server_root), 1, pldoc_http, doc_http).
 index((doc_latex), 3, pldoc_latex, doc_latex).
 index((latex_for_file), 3, pldoc_latex, doc_latex).
 index((latex_for_wiki_file), 3, pldoc_latex, doc_latex).
@@ -155,6 +235,7 @@ index((display), 2, edinburgh, edinburgh).
 index((unknown), 2, edinburgh, edinburgh).
 index((debug), 0, edinburgh, edinburgh).
 index((nodebug), 0, edinburgh, edinburgh).
+index((fileerrors), 2, edinburgh, edinburgh).
 index((edit), 1, prolog_edit, edit).
 index((edit), 0, prolog_edit, edit).
 index(('$editor_load_code'), 2, emacs_interface, emacs_interface).
@@ -173,10 +254,16 @@ index((existence_error), 2, error, error).
 index((permission_error), 3, error, error).
 index((instantiation_error), 1, error, error).
 index((representation_error), 1, error, error).
+index((syntax_error), 1, error, error).
 index((must_be), 2, error, error).
 index((is_of_type), 2, error, error).
 index((explain), 1, prolog_explain, explain).
 index((explain), 2, prolog_explain, explain).
+index((fast_read), 1, fastrw, fastrw).
+index((fast_write), 1, fastrw, fastrw).
+index((fast_read), 2, fastrw, fastrw).
+index((fast_write), 2, fastrw, fastrw).
+index((fast_write_to_string), 3, fastrw, fastrw).
 index((can_open_file), 2, files, files).
 index((chdir), 1, files, files).
 index((set_time_file), 3, files_ex, filesex).
@@ -184,11 +271,14 @@ index((reset_gensym), 0, gensym, gensym).
 index((reset_gensym), 1, gensym, gensym).
 index((gensym), 2, gensym, gensym).
 index((add_to_heap), 4, heaps, heaps).
+index((delete_from_heap), 4, heaps, heaps).
 index((empty_heap), 1, heaps, heaps).
 index((get_from_heap), 4, heaps, heaps).
 index((heap_size), 2, heaps, heaps).
 index((heap_to_list), 2, heaps, heaps).
+index((is_heap), 1, heaps, heaps).
 index((list_to_heap), 2, heaps, heaps).
+index((merge_heaps), 3, heaps, heaps).
 index((min_of_heap), 3, heaps, heaps).
 index((min_of_heap), 5, heaps, heaps).
 index((help), 1, online_help, help).
@@ -205,6 +295,7 @@ index((portray_clause), 2, prolog_listing, listing).
 index((member), 2, lists, lists).
 index((append), 2, lists, lists).
 index((append), 3, lists, lists).
+index((prefix), 2, lists, lists).
 index((select), 3, lists, lists).
 index((selectchk), 3, lists, lists).
 index((nextto), 3, lists, lists).
@@ -230,6 +321,7 @@ index((make), 0, make, make).
 index((new_memory_file), 1, memory_file, memfile).
 index((free_memory_file), 1, memory_file, memfile).
 index((size_memory_file), 2, memory_file, memfile).
+index((size_memory_file), 3, memory_file, memfile).
 index((open_memory_file), 3, memory_file, memfile).
 index((open_memory_file), 4, memory_file, memfile).
 index((atom_to_memory_file), 2, memory_file, memfile).
@@ -284,6 +376,10 @@ index((ord_union), 4, ordsets, ordsets).
 index((ord_subset), 2, ordsets, ordsets).
 index((ord_empty), 1, ordsets, ordsets).
 index((ord_memberchk), 2, ordsets, ordsets).
+index((ord_symdiff), 3, ordsets, ordsets).
+index((ord_member), 2, ordsets, ordsets).
+index((ord_seteq), 2, ordsets, ordsets).
+index((ord_intersection), 2, ordsets, ordsets).
 index((oset_is), 1, oset, oset).
 index((oset_union), 3, oset, oset).
 index((oset_int), 3, oset, oset).
@@ -299,6 +395,10 @@ index((pairs_keys), 2, pairs, pairs).
 index((group_pairs_by_key), 2, pairs, pairs).
 index((transpose_pairs), 2, pairs, pairs).
 index((map_list_to_pairs), 3, pairs, pairs).
+index((persistent), 1, persistency, persistency).
+index((db_attach), 2, persistency, persistency).
+index((db_sync), 1, persistency, persistency).
+index((db_sync_all), 1, persistency, persistency).
 index((doc_collect), 1, pldoc, pldoc).
 index((pldoc_loading), 0, pldoc, pldoc).
 index((set_test_options), 1, plunit, plunit).
@@ -314,6 +414,8 @@ index((porter_stem), 2, porter_stem, porter_stem).
 index((unaccent_atom), 2, porter_stem, porter_stem).
 index((tokenize_atom), 2, porter_stem, porter_stem).
 index((atom_to_stem_list), 2, porter_stem, porter_stem).
+index((portray_text), 1, portray_text, portray_text).
+index((set_portray_text), 2, portray_text, portray_text).
 index((process_create), 3, process, process).
 index((process_wait), 2, process, process).
 index((process_wait), 3, process, process).
@@ -352,10 +454,13 @@ index((xref_expand), 2, prolog_xref, prolog_xref).
 index((xref_source_file), 3, prolog_xref, prolog_xref).
 index((xref_source_file), 4, prolog_xref, prolog_xref).
 index((xref_public_list), 4, prolog_xref, prolog_xref).
+index((xref_public_list), 5, prolog_xref, prolog_xref).
 index((xref_meta), 2, prolog_xref, prolog_xref).
 index((xref_hook), 1, prolog_xref, prolog_xref).
 index((xref_used_class), 2, prolog_xref, prolog_xref).
 index((xref_defined_class), 3, prolog_xref, prolog_xref).
+index((protobuf_message), 2, protobufs, protobufs).
+index((protobuf_message), 3, protobufs, protobufs).
 index((phrase_from_file), 2, pure_input, pure_input).
 index((phrase_from_file), 3, pure_input, pure_input).
 index((stream_to_lazy_list), 2, pure_input, pure_input).
@@ -365,6 +470,8 @@ index((load_foreign_files), 3, qp_foreign, qpforeign).
 index((make_shared_object), 3, qp_foreign, qpforeign).
 index((make_foreign_wrapper_file), 1, qp_foreign, qpforeign).
 index((make_foreign_wrapper_file), 2, qp_foreign, qpforeign).
+index((make_foreign_resource_wrapper), 3, qp_foreign, qpforeign).
+index((load_foreign_resource), 2, qp_foreign, qpforeign).
 index((qsave_program), 1, qsave, qsave).
 index((qsave_program), 2, qsave, qsave).
 index((unix), 1, quintus, quintus).
@@ -387,7 +494,6 @@ index((sqrt), 2, quintus, quintus).
 index((genarg), 3, quintus, quintus).
 index((mode), 1, quintus, quintus).
 index((public), 1, quintus, quintus).
-index((meta_predicate), 1, quintus, quintus).
 index((no_style_check), 1, quintus, quintus).
 index((otherwise), 0, quintus, quintus).
 index((simple), 1, quintus, quintus).
@@ -423,7 +529,9 @@ index((rb_insert_new), 4, rbtrees, rbtrees).
 index((rb_delete), 3, rbtrees, rbtrees).
 index((rb_delete), 4, rbtrees, rbtrees).
 index((rb_visit), 2, rbtrees, rbtrees).
+index((rb_visit), 3, rbtrees, rbtrees).
 index((rb_keys), 2, rbtrees, rbtrees).
+index((rb_keys), 3, rbtrees, rbtrees).
 index((rb_map), 2, rbtrees, rbtrees).
 index((rb_map), 3, rbtrees, rbtrees).
 index((rb_partial_map), 4, rbtrees, rbtrees).
@@ -440,22 +548,6 @@ index((ord_list_to_rbtree), 2, rbtrees, rbtrees).
 index((is_rbtree), 1, rbtrees, rbtrees).
 index((rb_size), 2, rbtrees, rbtrees).
 index((rb_in), 3, rbtrees, rbtrees).
-index((load_rdf), 2, rdf, rdf).
-index((load_rdf), 3, rdf, rdf).
-index((xml_to_rdf), 3, rdf, rdf).
-index((process_rdf), 3, rdf, rdf).
-index((load_rdf_ntriples), 2, rdf_ntriples, rdf_ntriples).
-index((rdf_ntriple_part), 4, rdf_ntriples, rdf_ntriples).
-index((xml_to_plrdf), 3, rdf_parser, rdf_parser).
-index((element_to_plrdf), 3, rdf_parser, rdf_parser).
-index((rdf_name_space), 1, rdf_parser, rdf_parser).
-index((rdf_triples), 2, rdf_triple, rdf_triple).
-index((rdf_triples), 3, rdf_triple, rdf_triple).
-index((rdf_reset_ids), 0, rdf_triple, rdf_triple).
-index((rdf_start_file), 2, rdf_triple, rdf_triple).
-index((rdf_end_file), 1, rdf_triple, rdf_triple).
-index((anon_prefix), 1, rdf_triple, rdf_triple).
-index((rdf_write_xml), 2, rdf_write, rdf_write).
 index((readln), 1, readln, readln).
 index((readln), 2, readln, readln).
 index((readln), 5, readln, readln).
@@ -466,9 +558,7 @@ index((read_stream_to_codes), 3, read_util, readutil).
 index((read_file_to_codes), 3, read_util, readutil).
 index((read_file_to_terms), 3, read_util, readutil).
 index((record), 1, record, record).
-index((rewrite), 2, rewrite, rewrite).
-index((rew_term_expansion), 2, rewrite, rewrite).
-index((rew_goal_expansion), 2, rewrite, rewrite).
+index((current_record), 2, record, record).
 index((rlimit), 3, clib_rlimit, rlimit).
 index((setting), 4, settings, settings).
 index((setting), 2, settings, settings).
@@ -484,6 +574,8 @@ index((setting_property), 2, settings, settings).
 index((list_settings), 0, settings, settings).
 index((convert_setting_text), 3, settings, settings).
 index((sha_hash), 3, crypto_hash, sha).
+index((sha_new_ctx), 2, crypto_hash, sha).
+index((sha_hash_ctx), 4, crypto_hash, sha).
 index((hmac_sha), 4, crypto_hash, sha).
 index((ls), 0, shell, shell).
 index((ls), 1, shell, shell).
@@ -502,10 +594,15 @@ index((unload_foreign_library), 1, shlib, shlib).
 index((unload_foreign_library), 2, shlib, shlib).
 index((current_foreign_library), 2, shlib, shlib).
 index((reload_foreign_libraries), 0, shlib, shlib).
+index((use_foreign_library), 1, shlib, shlib).
+index((use_foreign_library), 2, shlib, shlib).
+index((snowball), 3, snowball, snowball).
+index((snowball_current_algorithm), 1, snowball, snowball).
 index((tcp_socket), 1, socket, socket).
 index((tcp_close_socket), 1, socket, socket).
 index((tcp_open_socket), 3, socket, socket).
 index((tcp_connect), 2, socket, socket).
+index((tcp_connect), 4, socket, socket).
 index((tcp_bind), 2, socket, socket).
 index((tcp_accept), 3, socket, socket).
 index((tcp_listen), 2, socket, socket).
@@ -518,14 +615,28 @@ index((udp_socket), 1, socket, socket).
 index((udp_receive), 4, socket, socket).
 index((udp_send), 4, socket, socket).
 index((predsort), 3, sort, sort).
-index((merge), 3, sort, sort).
-index((merge_set), 3, sort, sort).
 index((locale_sort), 2, sort, sort).
+index((load_certificate), 2, ssl, ssl).
+index((load_private_key), 3, ssl, ssl).
+index((load_public_key), 2, ssl, ssl).
+index((load_crl), 2, ssl, ssl).
+index((rsa_private_decrypt), 3, ssl, ssl).
+index((rsa_private_encrypt), 3, ssl, ssl).
+index((rsa_public_decrypt), 3, ssl, ssl).
+index((rsa_public_encrypt), 3, ssl, ssl).
+index((ssl_context), 3, ssl, ssl).
+index((ssl_init), 3, ssl, ssl).
+index((ssl_accept), 3, ssl, ssl).
+index((ssl_open), 3, ssl, ssl).
+index((ssl_open), 4, ssl, ssl).
+index((ssl_negotiate), 5, ssl, ssl).
+index((ssl_exit), 1, ssl, ssl).
 index((time), 1, prolog_statistics, statistics).
 index((profile), 1, prolog_statistics, statistics).
 index((profile), 3, prolog_statistics, statistics).
 index((show_profile), 2, prolog_statistics, statistics).
 index((show_profile), 1, prolog_statistics, statistics).
+index((stream_info), 1, stream_info, streaminfo).
 index((add_stream_to_pool), 2, stream_pool, streampool).
 index((delete_stream_from_pool), 1, stream_pool, streampool).
 index((close_stream_pool), 0, stream_pool, streampool).
@@ -564,6 +675,7 @@ index((subsumes), 2, terms, terms).
 index((subsumes_chk), 2, terms, terms).
 index((cyclic_term), 1, terms, terms).
 index((acyclic_term), 1, terms, terms).
+index((term_subsumer), 3, terms, terms).
 index((show_coverage), 1, prolog_cover, test_cover).
 index((covered_clauses), 4, prolog_cover, test_cover).
 index((make_tests), 3, test_wizard, test_wizard).
@@ -589,8 +701,11 @@ index((tnodebug), 1, thread_util, threadutil).
 index((tprofile), 1, thread_util, threadutil).
 index((alarm), 3, time, time).
 index((alarm), 4, time, time).
+index((alarm_at), 4, time, time).
 index((remove_alarm), 1, time, time).
 index((install_alarm), 1, time, time).
+index((install_alarm), 2, time, time).
+index((uninstall_alarm), 1, time, time).
 index((current_alarm), 4, time, time).
 index((call_with_time_limit), 2, time, time).
 index((tty_clear), 0, tty, tty).
@@ -621,6 +736,21 @@ index((pipe), 2, unix, unix).
 index((dup), 2, unix, unix).
 index((detach_IO), 0, unix, unix).
 index((environ), 1, unix, unix).
+index((uri_components), 2, uri, uri).
+index((uri_data), 3, uri, uri).
+index((uri_data), 4, uri, uri).
+index((uri_normalized), 2, uri, uri).
+index((uri_normalized_iri), 2, uri, uri).
+index((uri_normalized), 3, uri, uri).
+index((uri_normalized_iri), 3, uri, uri).
+index((uri_resolve), 3, uri, uri).
+index((uri_is_global), 1, uri, uri).
+index((uri_query_components), 2, uri, uri).
+index((uri_authority_components), 2, uri, uri).
+index((uri_authority_data), 3, uri, uri).
+index((uri_encoded), 3, uri, uri).
+index((uri_file_name), 2, uri, uri).
+index((uri_iri), 2, uri, uri).
 index((parse_url), 2, url, url).
 index((parse_url), 3, url, url).
 index((is_absolute_url), 1, url, url).
@@ -630,6 +760,9 @@ index((www_form_encode), 2, url, url).
 index((parse_url_search), 2, url, url).
 index((url_iri), 2, url, url).
 index((file_name_to_url), 2, url, url).
+index((set_url_encoding), 2, url, url).
+index((utf8_codes), 3, utf8, utf8).
+index((vm_list), 1, prolog_vm, vm).
 index((when), 2, when, when).
 index((www_open_url), 1, www_browser, www_browser).
 index((expand_url_path), 2, www_browser, www_browser).
