@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2009, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -33,21 +33,27 @@
 	  [ doc_server/1,		% ?Port
 	    doc_server/2,		% ?Port, +Options
 	    doc_browser/0,
-	    doc_browser/1,		% +What
-	    doc_server_root/1		% -Root
+	    doc_browser/1		% +What
 	  ]).
 :- use_module(library(pldoc)).
-:- use_module(library('http/thread_httpd')).
-:- use_module(library('http/http_parameters')).
-:- use_module(library('http/html_write')).
-:- use_module(library('http/mimetype')).
-:- use_module(library('http/dcg_basics')).
+:- use_module(library(http/thread_httpd)).
+:- use_module(library(http/http_parameters)).
+:- use_module(library(http/html_write)).
+:- use_module(library(http/mimetype)).
+:- use_module(library(http/dcg_basics)).
+:- use_module(library(http/http_dispatch)).
+:- use_module(library(http/http_log)).
+:- use_module(library(http/http_hook)).
+:- use_module(library(http/http_path)).
+:- use_module(library(http/http_wrapper)).
+:- use_module(library(uri)).
 :- use_module(library(debug)).
 :- use_module(library(lists)).
 :- use_module(library(url)).
 :- use_module(library(socket)).
 :- use_module(library(option)).
 :- use_module(library(error)).
+:- use_module(library(www_browser)).
 :- use_module(pldoc(doc_process)).
 :- use_module(pldoc(doc_htmlsrc)).
 :- use_module(pldoc(doc_html)).
@@ -56,6 +62,7 @@
 :- use_module(pldoc(doc_man)).
 :- use_module(pldoc(doc_wiki)).
 :- use_module(pldoc(doc_util)).
+:- use_module(pldoc(doc_access)).
 
 /** <module> Documentation server
 
@@ -64,8 +71,15 @@ server that allows for browsing the   documentation  of all files loaded
 _after_ library(pldoc) has been loaded.
 */
 
-:- multifile
-	log_hook/3.			% +Port, +ReqNr, +Result
+:- dynamic
+	doc_server_port/1.
+
+http:location(pldoc, root(.), []).
+http:location(pldoc_man, pldoc(refman), []).
+http:location(pldoc_pkg, pldoc(package), []).
+http:location(pldoc_resource, Path, []) :-
+	http_location_by_id(pldoc_resource, Path).
+
 
 %%	doc_server(?Port) is det.
 %%	doc_server(?Port, +Options) is det.
@@ -74,7 +88,7 @@ _after_ library(pldoc) has been loaded.
 %	server is started in a seperate   thread.  Options are handed to
 %	http_server/2.  In  addition,   the    following   options   are
 %	recognised:
-%	
+%
 %		* allow(HostOrIP)
 %		Allow connections from HostOrIP.  If HostOrIP is an atom
 %		it is matched to the hostname.  It if starts with a .,
@@ -84,17 +98,14 @@ _after_ library(pldoc) has been loaded.
 %
 %		* deny(HostOrIP)
 %		See allow(HostOrIP).
-%		
+%
 %		* edit(Bool)
 %		Allow editing from localhost connections? Default:
 %		=true=.
-%		
-%		* root(Path)
-%		Path of the root.  Default is /
-%	
+%
 %	The predicate doc_server/1 is defined as below, which provides a
 %	good default for development.
-%	
+%
 %	==
 %	doc_server(Port) :-
 %		doc_server(Port,
@@ -102,10 +113,9 @@ _after_ library(pldoc) has been loaded.
 %			     allow(localhost)
 %			   ]).
 %	==
-%	
+%
 %	@see	doc_browser/1
 
-
 doc_server(Port) :-
 	doc_server(Port,
 		   [ workers(1),
@@ -117,28 +127,37 @@ doc_server(Port, _) :-
 	catch(doc_current_server(Port), _, fail), !.
 doc_server(Port, Options) :-
 	prepare_editor,
-	auth_options(Options, Options1),
-	root_option(Options1, Options2),
-	edit_option(Options2, ServerOptions),
-	append(ServerOptions,		% Put provides options first,
-	       [ port(Port),		% so they override our defaults
-		 timeout(60),
-		 keep_alive_timeout(1),
-		 local(4000),		% keep stack sizes independent
-		 global(4000),		% from main application
-		 trail(4000)
-	       ], HTTPOptions),
-	http_server(doc_reply, HTTPOptions),
-	call_log_hook(started, 0, port(Port)),
+	host_access_options(Options, ServerOptions),
+	merge_options(ServerOptions,
+		      [ port(Port),
+			timeout(60),
+			keep_alive_timeout(1),
+			local(4000),	% small stacks for now
+			global(4000),
+			trail(4000)
+		      ], HTTPOptions),
+	http_server(http_dispatch, HTTPOptions),
+	assert(doc_server_port(Port)),
 	print_message(informational, pldoc(server_started(Port))).
 
+%%	doc_current_server(-Port) is det.
+%
+%	TCP/IP port of the documentation server.   Fails if no server is
+%	running. Note that in the current   infrastructure we can easily
+%	be embedded into another  Prolog  HTTP   server.  If  we are not
+%	started from doc_server/2, we  return  the   port  of  a running
+%	HTTP server.
+%
+%	@tbd	Trap destruction of the server.
+%	@error	existence_error(http_server, pldoc)
+
 doc_current_server(Port) :-
-	http_current_server(doc_reply, Port), !.
-doc_current_server(Port) :-
-	findall(P, http_current_server(_, P), Ports),
-	Ports = [Port], !.		% only one; hope its the right one
-doc_current_server(_) :-
-	existence_error(http_server, doc_reply).
+	(   doc_server_port(P)
+	->  Port = P
+	;   http_current_server(_:_, P)
+	->  Port = P
+	;   existence_error(http_server, pldoc)
+	).
 
 %%	doc_browser is det.
 %%	doc_browser(+What) is semidet.
@@ -154,7 +173,7 @@ doc_browser(Spec) :-
 	www_open_url(URL).
 
 browser_url([], Root) :- !,
-	doc_server_root(Root).
+	http_location_by_id(pldoc_root, Root).
 browser_url(Name, URL) :-
 	atom(Name), !,
 	browser_url(Name/_, URL).
@@ -166,22 +185,18 @@ browser_url(Name//Arity, URL) :-
 browser_url(Name/Arity, URL) :- !,
 	must_be(atom, Name),
 	(   predicate(Name, Arity, _, _, _)
-	->  doc_server_root(Root),
-	    format(string(S), '~q/~w', [Name, Arity]),
-	    www_form_encode(S, Enc),
-	    format(string(URL), '~wman?predicate=~w', [Root, Enc])
+	->  format(string(S), '~q/~w', [Name, Arity]),
+	    http_link_to_id(pldoc_man, [predicate=S], URL)
 	;   browser_url(_:Name/Arity, URL)
 	).
 browser_url(Spec, URL) :- !,
 	Spec = M:Name/Arity,
 	doc_comment(Spec, _Pos, _Summary, _Comment), !,
-	doc_server_root(Root),
 	(   var(M)
 	->  format(string(S), '~q/~w', [Name, Arity])
 	;   format(string(S), '~q:~q/~w', [M, Name, Arity])
 	),
-	www_form_encode(S, Enc),
-	format(string(URL), '~wdoc_for?object=~w', [Root, Enc]).
+	http_link_to_id(doc_object, [object=S], URL).
 
 %%	prepare_editor
 %
@@ -194,223 +209,74 @@ prepare_editor :-
 prepare_editor.
 
 
-doc_reply(Request) :-
-	flag('$pldoc_current_request', N, N+1),
-	call_log_hook(enter, N, Request),
-	call_cleanup(do_reply(Request), Why,
-		     call_log_hook(exit, N, Why)).
-
-do_reply(Request) :-
-	memberchk(peer(Peer), Request),
-	select(path(Path0), Request, Request1),
-	(   allowed_peer(Peer)
-	->  debug(pldoc, 'HTTP ~q', [Path0]),
-	    (	normalise_path(Path0, Path),
-		reply(Path, [path(Path)|Request1])
-	    ->	true
-	    ;	throw(http_reply(not_found(Path0)))
-	    )		      
-	;   throw(http_reply(forbidden(Path0)))
-	).
-
-
-%%	call_log_hook(+Port, +ReqNR, +Data)
-%
-%	Call log_hook/3, but always succeed.
-
-call_log_hook(Port, ReqNR, Data) :-
-	log_hook(Port, ReqNR, Data), !.
-call_log_hook(_, _, _).
-
-
-		 /*******************************
-		 *	  ACCESS CONTROL	*
-		 *******************************/
-
-:- dynamic
-	allow_from/1,
-	deny_from/1.
-
-%%	auth_options(+AllOptions, -NoAuthOptions) is det.
-%
-%	Filter the authorization options from   AllOptions,  leaving the
-%	remaining options in NoAuthOptions.
-
-auth_options([], []).
-auth_options([H|T0], T) :-
-	auth_option(H), !,
-	auth_options(T0, T).
-auth_options([H|T0], [H|T]) :-
-	auth_options(T0, T).
-
-auth_option(allow(From)) :-
-	assert(allow_from(From)).
-auth_option(deny(From)) :-
-	assert(deny_from(From)).
-
-%%	match_peer(:RuleSet, +PlusMin, +Peer) is semidet.
-%
-%	True if Peer is covered by the   ruleset RuleSet. Peer is a term
-%	ip(A,B,C,D). RuleSet is a predicate with   one  argument that is
-%	either  a  partial  ip  term,  a    hostname  or  a  domainname.
-%	Domainnames start with a '.'.
-%	
-%	@param PlusMin	Positive/negative test.  If IP->Host fails, a
-%		       	positive test fails, while a negative succeeds.
-%			I.e. deny('.com') succeeds for unknown IP
-%			addresses.
-
-match_peer(Spec, _, Peer) :-
-	call(Spec, Peer), !.
-match_peer(Spec, PM, Peer) :-
-	(   call(Spec, HOrDom), atom(HOrDom)
-	->  (   catch(tcp_host_to_address(Host, Peer), E, true),
-	        var(E)
-	    ->	call(Spec, HostOrDomain),
-		atom(HostOrDomain),
-		(   sub_atom(HostOrDomain, 0, _, _, '.')
-		->  sub_atom(Host, _, _, 0, HostOrDomain)
-		;   HostOrDomain == Host
-		)
-	    ;   PM == (+)
-	    ->	!, fail
-	    ;	true
-	    )
-	).
-	
-%%	allowed_peer(+Peer) is semidet.
-%
-%	True if Peer is allowed according to the rules.
-
-allowed_peer(Peer) :-
-	match_peer(deny_from, -, Peer), !,
-	match_peer(allow_from, +, Peer).
-allowed_peer(Peer) :-
-	allow_from(_), !,
-	match_peer(allow_from, +, Peer).
-allowed_peer(_).
-
-
-:- dynamic
-	can_edit/1.
-
-%%	allow_edit(+Request) is semidet.
-%
-%	True if, given Request, we allow editing sources.
-
-allow_edit(_) :-
-	can_edit(false), !, 
-	fail.
-allow_edit(Request) :-
-	(   memberchk(x_forwarded_for(Forwarded), Request),
-	    primary_forwarded_host(Forwarded, IPAtom),
-	    parse_ip(IPAtom, Peer)
-	->  true
-	;   memberchk(peer(Peer), Request)
-	),
-	match_peer(localhost, +, Peer).
-
-
-%%	primary_forwarded_host(+Spec, -Host) is det.
-%
-%	x_forwarded host contains multiple hosts seperated   by  ', ' if
-%	there are multiple proxy servers in   between.  The first one is
-%	the one the user's browser knows about.
-
-primary_forwarded_host(Spec, Host) :-
-	sub_atom(Spec, B, _, _, ','), !,
-	sub_atom(Spec, 0, B, _, Host).
-primary_forwarded_host(Host, Host).
-
-
-localhost(ip(127,0,0,1)).
-localhost(localhost).
-
-parse_ip(Atom, IP) :-
-	atom_codes(Atom, Codes),
-	phrase(ip(IP), Codes).
-
-%%	ip(?IP)// is semidet.
-%
-%	Parses A.B.C.D into ip(A,B,C,D)
-
-ip(ip(A,B,C,D)) -->
-	integer(A), ".", integer(B), ".", integer(C), ".", integer(D).
-	
-edit_option(Options0, Options) :-
-	select_option(edit(Bool), Options0, Options), !,
-	assert(can_edit(Bool)).
-edit_option(Options, Options).
-
-
-		 /*******************************
-		 *	       ROOT		*
-		 *******************************/
-
-:- dynamic
-	root/1.
-
-root_option(Options0, Options) :-
-	select_option(root(Root), Options0, Options), !,
-	assert(root(Root)).
-root_option(Options, Options).
-	
-%%	doc_server_root(?Root) is semidet.
-%
-%	True if Root is the root of our documentation server. Default is
-%	=|/|=. Can be set with the =root= option of doc_server/1.
-
-doc_server_root(Root) :-
-	(   root(Root0)
-	->  Root = Root0
-	;   Root = /
-	).
-
-
-%%	normalise_path(+Path0, -NormalPath) is det.
-%
-%	Make paths relative to / if it was moved.
-
-normalise_path(Path0, Path) :-
-	(   doc_server_root(/)
-	->  Path = Path0
-	;   doc_server_root(Root),
-	    atom_concat(Root, Rest, Path0)
-	->  atom_concat(/, Rest, Path)
-	).
-
-
 		 /*******************************
 		 *	    USER REPLIES	*
 		 *******************************/
 
-:- discontiguous
-	reply/2.
+:- http_handler(pldoc(.),	   pldoc_root,
+		[prefix, authentication(pldoc(read))]).
+:- http_handler(pldoc('index.html'), pldoc_index,   []).
+:- http_handler(pldoc(file),	   pldoc_file,	   []).
+:- http_handler(pldoc(directory),  pldoc_dir,	   []).
+:- http_handler(pldoc(edit),	   pldoc_edit,
+		[authentication(pldoc(edit))]).
+:- http_handler(pldoc(doc),	   pldoc_doc,	   [prefix]).
+:- http_handler(pldoc(man),	   pldoc_man,	   []).
+:- http_handler(pldoc(doc_for),	   pldoc_object,   []).
+:- http_handler(pldoc(search),	   pldoc_search,   []).
+:- http_handler(pldoc('res/'),	   pldoc_resource, [prefix]).
 
-%	/
-%	
+
+%%	pldoc_root(+Request)
+%
 %	Reply using the index-page  of   the  Prolog  working directory.
 %	There are various options for the   start directory. For example
 %	we could also use the file or   directory of the file that would
 %	be edited using edit/0.
 
-reply(/, _) :-
+pldoc_root(Request) :-
+	http_parameters(Request,
+			[ empty(Empty, [ oneof([true,false]),
+					 default(false)
+				       ])
+			]),
+	pldoc_root(Request, Empty).
+
+pldoc_root(Request, false) :-
+	http_location_by_id(pldoc_root, Root),
+	memberchk(path(Path), Request),
+	Root \== Path, !,
+	existence_error(http_location, Path).
+pldoc_root(_Request, false) :-
 	working_directory(Dir0, Dir0),
 	allowed_directory(Dir0), !,
 	ensure_slash_end(Dir0, Dir1),
 	doc_file_href(Dir1, Ref0),
 	atom_concat(Ref0, 'index.html', Index),
 	throw(http_reply(see_other(Index))).
-reply(/, _) :-
-	reply_page('PlDoc directory index',
-		   \doc_links('', [])).
+pldoc_root(Request, _) :-
+	pldoc_index(Request).
 
 
-%	/file?file=REF
-%	
-%	Reply using documentation of file
+%%	pldoc_index(+Request)
+%
+%	HTTP handle for /index.html, providing an overall overview
+%	of the available documentation.
 
-reply('/file', Request) :-
+pldoc_index(_Request) :-
+	reply_html_page(pldoc(index),
+			title('SWI-Prolog documentation'),
+			[ \doc_links('', []),
+			   h1('SWI-Prolog documentation'),
+			  \man_overview([])
+			]).
+
+
+%%	pldoc_file(+Request)
+%
+%	Hander for /file?file=File, providing documentation for File.
+
+pldoc_file(Request) :-
 	http_parameters(Request,
 			[ file(File, [])
 			]),
@@ -418,22 +284,49 @@ reply('/file', Request) :-
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
-	format('Content-type: text/html~n~n'),
-	doc_for_file(File, current_output, []).
+	doc_for_file(File, []).
 
-%	/edit?file=REF
-%	
-%	Start SWI-Prolog editor on file
+%%	pldoc_edit(+Request)
+%
+%	HTTP handler that starts the user's   default editor on the host
+%	running the server. This  handler  can   only  accessed  if  the
+%	browser connection originates from  =localhost=.   The  call can
+%	edit files using the =file=  attribute   or  a predicate if both
+%	=name= and =arity= is given and optionally =module=.
 
-reply('/edit', Request) :-
-	allow_edit(Request), !,
+pldoc_edit(Request) :-
+	http:authenticate(pldoc(edit), Request, _),
 	http_parameters(Request,
-			[ file(File,     [optional(true)]),
-			  module(Module, [optional(true)]),
-			  name(Name,     [optional(true)]),
-			  arity(Arity,   [integer, optional(true)])
+			[ file(File,
+			       [ optional(true),
+				 description('Name of the file to edit')
+			       ]),
+			  line(Line,
+			       [ optional(true),
+				 integer,
+				 description('Line in the file')
+			       ]),
+			  name(Name,
+			       [ optional(true),
+				 description('Name of a Prolog predicate to edit')
+			       ]),
+			  arity(Arity,
+				[ integer,
+				  optional(true),
+				  description('Arity of a Prolog predicate to edit')
+				]),
+			  module(Module,
+				 [ optional(true),
+				   description('Name of a Prolog module to search for predicate')
+				 ])
 			]),
 	(   atom(File)
+	->  allowed_file(File)
+	;   true
+	),
+	(   atom(File), integer(Line)
+	->  Edit = file(File, line(Line))
+	;   atom(File)
 	->  Edit = file(File)
 	;   atom(Name), integer(Arity)
 	->  (   atom(Module)
@@ -441,20 +334,20 @@ reply('/edit', Request) :-
 	    ;	Edit = (Name/Arity)
 	    )
 	),
-	format(string(Cmd), '~q', [edit(Edit)]),
 	edit(Edit),
-	reply_page('Edit',
-		   [ p(['Started ', Cmd])
-		   ]).
-reply('/edit', _Request) :-
-	throw(http_reply(forbidden('/edit'))).
+	format('Content-type: text/plain~n~n'),
+	format('Started ~q~n', [edit(Edit)]).
+pldoc_edit(_Request) :-
+	http_location_by_id(pldoc_edit, Location),
+	throw(http_reply(forbidden(Location))).
 
 
-%	/directory?dir=Dir
-%	
-%	Give index of directory.  Mapped to /doc/Dir/index.html.
+%%	pldoc_dir(+Request)
+%
+%	Handler for /directory?dir=Dir, providing an index for
+%	Dir.  Mapped to /doc/Dir/index.html.
 
-reply('/directory', Request) :-
+pldoc_dir( Request) :-
 	http_parameters(Request,
 			[ dir(Dir0, [])
 			]),
@@ -476,6 +369,8 @@ allowed_directory(Dir) :-
 allowed_directory(Dir) :-
 	working_directory(CWD, CWD),
 	same_file(CWD, Dir).
+allowed_directory(Dir) :-
+	prolog:doc_directory(Dir).
 
 
 %%	allowed_file(+File) is semidet.
@@ -491,17 +386,48 @@ allowed_file(File) :-
 	allowed_directory(Dir).
 
 
-%	/doc/Path
-%	
+%%	pldoc_resource(+Request)
+%
+%	Handler for /res/File, serving CSS, JS and image files.
+
+pldoc_resource(Request) :-
+	http_location_by_id(pldoc_resource, ResRoot),
+	memberchk(path(Path), Request),
+	atom_concat(ResRoot, File, Path),
+	file(File, Local),
+	http_reply_file(pldoc(Local), [], Request).
+
+file('pldoc.css',     'pldoc.css').
+file('pllisting.css', 'pllisting.css').
+file('pldoc.js',      'pldoc.js').
+file('edit.gif',      'edit.gif').
+file('up.gif',        'up.gif').
+file('source.png',    'source.png').
+file('public.png',    'public.png').
+file('private.png',   'private.png').
+file('reload.gif',    'reload.gif').
+file('favicon.ico',   'favicon.ico').
+file('h1-bg.png',     'h1-bg.png').
+file('pub-bg.png',    'pub-bg.png').
+file('priv-bg.png',   'priv-bg.png').
+file('multi-bg.png',  'multi-bg.png').
+
+
+%%	pldoc_doc(+Request)
+%
+%	Handler for /doc/Path
+%
 %	Reply documentation of a file. Path is  the absolute path of the
 %	file for which to return the  documentation. Extension is either
 %	none, the Prolog extension or the HTML extension.
-%	
+%
 %	Note that we reply  with  pldoc.css   if  the  file  basename is
 %	pldoc.css to allow for a relative link from any directory.
 
-reply(ReqPath, Request) :-
-	atom_concat('/doc', AbsFile0, ReqPath),
+pldoc_doc(Request) :-
+	memberchk(path(ReqPath), Request),
+	http_location_by_id(pldoc_doc, Me),
+	atom_concat(Me, AbsFile0, ReqPath),
 	(   sub_atom(ReqPath, _, _, 0, /)
 	->  atom_concat(ReqPath, 'index.html', File),
 	    throw(http_reply(moved(File)))
@@ -511,37 +437,32 @@ reply(ReqPath, Request) :-
 	->  documentation(AbsFile, Request)
 	).
 
-documentation(Path, _Request) :-
+documentation(Path, Request) :-
 	file_base_name(Path, Base),
 	file(_, Base), !,			% serve pldoc.css, etc.
-	reply_file(pldoc(Base)).
+	http_reply_file(pldoc(Base), [], Request).
 documentation(Path, Request) :-
 	Index = '/index.html',
-	sub_atom(Path, _, _, 0, Index), 
+	sub_atom(Path, _, _, 0, Index),
 	atom_concat(Dir, Index, Path),
 	exists_directory(Dir), !,		% Directory index
 	(   allowed_directory(Dir)
 	->  edit_options(Request, EditOptions),
-	    format('Content-type: text/html~n~n'),
-	    doc_for_dir(Dir, current_output, EditOptions)
+	    doc_for_dir(Dir, EditOptions)
 	;   throw(http_reply(forbidden(Dir)))
 	).
 documentation(File, _Request) :-
-	(   file_name_extension(_, txt, File)
-	;   file_base_name(File, Base),
-	    autolink_file(Base, wiki)
-	),
-	(   allowed_file(File)
+	wiki_file(File, WikiFile), !,
+	(   allowed_file(WikiFile)
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
-	format('Content-type: text/html~n~n'),
-	doc_for_wiki_file(File, current_output, []).
+	doc_for_wiki_file(WikiFile, []).
 documentation(Path, Request) :-
 	http_parameters(Request,
 			[ public_only(Public),
 			  reload(Reload),
-			  source(Source)
+			  show(Show)
 			],
 			[ attribute_declarations(param)
 			]),
@@ -550,28 +471,57 @@ documentation(Path, Request) :-
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
-	(   Reload == true
+	(   Reload == true,
+	    source_file(File)
 	->  load_files(File, [if(changed), imports([])])
 	;   true
 	),
 	edit_options(Request, EditOptions),
-	format('Content-type: text/html~n~n'),
-	(   Source == true
-	->  source_to_html(File, stream(current_output), [])
-	;   doc_for_file(File, current_output,
+	(   Show == src
+	->  format('Content-type: text/html~n~n', []),
+	    source_to_html(File, stream(current_output),
+			   [ skin(src_skin(Request, Show))
+			   ])
+	;   Show == raw
+	->  http_reply_file(File,
+			    [ unsafe(true), % is already validated
+			      mime_type(text/plain)
+			    ], Request)
+	;   doc_for_file(File,
 			 [ public_only(Public)
 			 | EditOptions
 			 ])
 	).
 
 
+src_skin(Request, _Show, header, Out) :- !,
+	memberchk(request_uri(ReqURI), Request), !,
+	replace_parameters(ReqURI, [show(raw)], RawLink),
+	phrase(html(div(class(src_formats),
+			[ 'View source as ', a(href(RawLink), raw)
+			])), Tokens),
+	print_html(Out, Tokens).
+
+replace_parameters(ReqURI, Extra, URI) :-
+	uri_components(ReqURI, C0),
+	uri_data(search, C0, Search0),
+	(   var(Search0)
+	->  uri_query_components(Search, Extra)
+	;   uri_query_components(Search0, Form0),
+	    merge_options(Extra, Form0, Form),
+	    uri_query_components(Search, Form)
+	),
+	uri_data(search, C0, Search, C),
+	uri_components(URI, C).
+
+
 %%	edit_options(+Request, -Options) is det.
 %
 %	Return edit(true) in Options  if  the   connection  is  from the
 %	localhost.
 
 edit_options(Request, [edit(true)]) :-
-	allow_edit(Request), !.
+	catch(http:authenticate(pldoc(edit), Request, _), _, fail), !.
 edit_options(_, []).
 
 
@@ -587,6 +537,23 @@ pl_file(File, PlFile) :-
 			   ], PlFile).
 pl_file(File, File).
 
+%%	wiki_file(+File, -TxtFile) is semidet.
+%
+%	True if TxtFile is an existing file  that must be served as wiki
+%	file.
+
+wiki_file(File, TxtFile) :-
+	file_name_extension(_, txt, File), !,
+	TxtFile = File.
+wiki_file(File, TxtFile) :-
+	file_base_name(File, Base),
+	autolink_file(Base, wiki), !,
+	TxtFile = File.
+wiki_file(File, TxtFile) :-
+	file_name_extension(Base, html, File),
+	file_name_extension(Base, txt, TxtFile),
+	access_file(TxtFile, read).
+
 
 %%	clean_path(+AfterDoc, -AbsPath)
 %
@@ -599,46 +566,36 @@ clean_path(Path0, Path) :-
 clean_path(Path, Path).
 
 
-%	/pldoc.css
-%	
-%	Reply the documentation style-sheet.
+%%	pldoc_man(+Request)
+%
+%	Handler for /man, offerring one of the parameters:
+%
+%	    * predicate=PI
+%	    providing documentation from the manual on the predicate PI.
+%	    * 'CAPI'=F
+%	    providing documentation from the manual on the C-function F.
 
-reply(Path, _Request) :-
-	file(Path, LocalFile),
-	reply_file(pldoc(LocalFile)).
-
-file('/pldoc.css',     'pldoc.css').
-file('/pllisting.css', 'pllisting.css').
-file('/pldoc.js',      'pldoc.js').
-file('/edit.gif',      'edit.gif').
-file('/up.gif',	       'up.gif').
-file('/source.gif',    'source.gif').
-file('/zoomin.gif',    'zoomin.gif').
-file('/zoomout.gif',   'zoomout.gif').
-file('/reload.gif',    'reload.gif').
-file('/favicon.ico',   'favicon.ico').
-
-
-%	/man?predicate=PI
-%	
-%	Provide documentation from the manual.
-%	
-%	@tbd	Make link to reference manual.
-
-reply('/man', Request) :-
+pldoc_man(Request) :-
 	http_parameters(Request,
-			[ predicate(PI, [])
+			[ predicate(PI, [optional(true)]),
+			  'CAPI'(F,     [optional(true)])
 			]),
 	format(string(Title), 'Manual -- ~w', [PI]),
-	reply_page(Title,
-		   [ \man_page(PI, [])
-		   ]).
+	(   ground(PI)
+	->  Obj = PI
+	;   ground(F)
+	->  Obj = c(F)
+	),
+	reply_html_page(pldoc(man),
+			title(Title),
+			\man_page(Obj, [])).
 
-%	/doc_for?object=Term
-%	
-%	Provide documentation for the given term
+%%	pldoc_object(+Request)
+%
+%	Handler for /doc_for?object=Term, Provide  documentation for the
+%	given term.
 
-reply('/doc_for', Request) :-
+pldoc_object(Request) :-
 	http_parameters(Request,
 			[ object(Atom, [])
 			]),
@@ -648,72 +605,47 @@ reply('/doc_for', Request) :-
 	;   Title = Atom
 	),
 	edit_options(Request, EditOptions),
-	reply_page(Title,
-		   [ \object_page(Obj, EditOptions)
-		   ]).
+	reply_html_page(pldoc(object),
+			title(Title),
+			\object_page(Obj, EditOptions)).
 
 
-%	/search?for=String
-%	
-%	Search for String
+%%	pldoc_search(+Request)
+%
+%	Search the collected PlDoc comments and Prolog manual.
 
-reply('/search', Request) :-
+pldoc_search(Request) :-
 	http_parameters(Request,
-			[ for(For, [length > 1]),
+			[ for(For,
+			      [ length > 1,
+				description('String to search for')
+			      ]),
 			  in(In,
 			     [ oneof([all,app,man]),
-			       default(all)
+			       default(all),
+			       description('Search everying, application only or manual only')
 			     ]),
 			  match(Match,
 				[ oneof([name,summary]),
-				  default(summary)
+				  default(summary),
+				  description('Match only the name or also the summary')
 				]),
-			  resultFormat(Format, [ oneof(long,summary),
-						 default(summary)
-					       ])
+			  resultFormat(Format,
+				       [ oneof(long,summary),
+					 default(summary),
+					 description('Return full documentation or summary-lines')
+				       ])
 			]),
 	edit_options(Request, EditOptions),
 	format(string(Title), 'Prolog search -- ~w', [For]),
-	reply_page(Title,
-		   [ \search_reply(For,
-				   [ resultFormat(Format),
-				     search_in(In),
-				     search_match(Match)
-				   | EditOptions
-				   ])
-		   ]).
-
-%	/package/Name
-%	
-%	Show documentation file of a package.  Exploits the file
-%	search path =package_documentation=.
-
-reply(Path, _Request) :-
-	atom_concat('/package/', Package, Path), !,
-	absolute_file_name(package_documentation(Package),
-			   DocFile,
-			   [ access(read),
-			     file_errors(fail)
-			   ]),
-	reply_file(DocFile).
-
-
-
-		 /*******************************
-		 *	       UTIL		*
-		 *******************************/
-
-reply_page(Title, Content) :-
-	doc_page_dom(Title, Content, DOM),
-	phrase(html(DOM), Tokens),
-	format('Content-type: text/html~n~n'),
-	print_html_head(current_output),
-	print_html(Tokens).
-
-reply_file(File) :-
-	absolute_file_name(File, Path, [access(read)]),
-	file_mime_type(Path, MimeType),
-	throw(http_reply(file(MimeType, Path))).
+	reply_html_page(pldoc(search),
+			title(Title),
+			\search_reply(For,
+				      [ resultFormat(Format),
+					search_in(In),
+					search_match(Match)
+				      | EditOptions
+				      ])).
 
 
 		 /*******************************
@@ -721,16 +653,19 @@ reply_file(File) :-
 		 *******************************/
 
 param(public_only,
-      [ oneof([true,false]),
-	default(true)
+      [ boolean,
+	default(true),
+	description('If true (default), hide private predicates')
       ]).
 param(reload,
-      [ oneof([true,false]),
-	default(false)
+      [ boolean,
+	default(false),
+	description('Reload the file and its documentation')
       ]).
-param(source,
-      [ oneof([true,false]),
-	default(false)
+param(show,
+      [ oneof([doc,src,raw]),
+	default(doc),
+	description('How to show the file')
       ]).
 
 
@@ -742,23 +677,7 @@ param(source,
 	prolog:message/3.
 
 prolog:message(pldoc(server_started(Port))) -->
+	{ http_location_by_id(pldoc_root, Root) },
 	[ 'Started Prolog Documentation server at port ~w'-[Port], nl,
-	  'You may access the server at http://localhost:~w/'-[Port]
+	  'You may access the server at http://localhost:~w~w'-[Port, Root]
 	].
-
-
-                 /*******************************
-                 *        PCEEMACS SUPPORT      *
-                 *******************************/
-
-:- multifile
-        emacs_prolog_colours:goal_colours/2,
-        prolog:called_by/2.
-
-
-emacs_prolog_colours:goal_colours(reply_page(_, HTML),
-                                  built_in-[classify, Colours]) :-
-        catch(html_write:html_colours(HTML, Colours), _, fail).
-
-prolog:called_by(reply_page(_, HTML), Called) :-
-        catch(phrase(html_write:called_by(HTML), Called), _, fail).
