@@ -1,11 +1,12 @@
 /*  $Id$
 
-    Part of XPCE --- The SWI-Prolog GUI toolkit
+    Part of SWI-Prolog
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        wielemak@science.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2007, University of Amsterdam
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org
+    Copyright (C): 1985-2010, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -31,6 +32,7 @@
 
 :- module(html_write,
 	  [ reply_html_page/2,		% :Head, :Body
+	    reply_html_page/3,		% +Style, :Head, :Body
 
 					% Basic output routines
 	    page//1,			% :Content
@@ -56,23 +58,30 @@
 					% Emitting the HTML code
 	    print_html/1,		% +List
 	    print_html/2,		% +Stream, +List
-	    html_print_length/2		% +List, -Length
+	    html_print_length/2,	% +List, -Length
+
+					% Extension support
+	    (html_meta)/1,		% +Spec
+	    op(1150, fx, html_meta)
 	  ]).
 :- use_module(library(error)).
+:- use_module(library(apply)).
 :- use_module(library(lists)).
 :- use_module(library(option)).
 :- use_module(library(pairs)).
 :- use_module(library(sgml)).		% Quote output
+:- use_module(library(url)).
 :- use_module(library(quintus)).	% for meta_predicate/1
 :- set_prolog_flag(generate_debug_info, false).
 
 :- meta_predicate
+	reply_html_page(+, :, :),
 	reply_html_page(:, :),
 	html(:, -, +),
 	page(:, -, +),
 	page(:, :, -, +),
-	pagehead(:, -, +),
-	pagebody(:, -, +),
+	pagehead(+, :, -, +),
+	pagebody(+, :, -, +),
 	html_receive(+, 3, -, +),
 	html_post(+, :, -, +).
 
@@ -116,17 +125,17 @@ encoding.
 %	flags to ensure  with  proper   multi-threaded  behaviour  where
 %	setting an option is local to the   thread and new threads start
 %	with the options from the parent thread.  Defined options are:
-%	
+%
 %		* dialect(Dialect)
 %		One of =html= (default) or =xhtml=.
-%		
+%
 %		* doctype(+DocType)
 %		Set the =|<|DOCTYPE|= DocType =|>|= line for page//1 and
 %		page//2.
-%		
+%
 %		* content_type(+ContentType)
-%		Set the =|Content-type|= for reply_html_page/2
-%		
+%		Set the =|Content-type|= for reply_html_page/3
+%
 %	Note  that  the  doctype  is  covered    by  two  prolog  flags:
 %	=html_doctype= for the html dialect  and =xhtml_doctype= for the
 %	xhtml dialect. Dialect muct be switched before doctype.
@@ -196,14 +205,13 @@ init_options :-
 	(   option_default(Name, Value),
 	    (	current_prolog_flag(Name, _)
 	    ->	true
-	    ;	set_prolog_flag(Name, Value)
+	    ;	create_prolog_flag(Name, Value, [])
 	    ),
 	    fail
 	;   true
 	).
-	
-:- initialization
-	init_options.
+
+:- init_options.
 
 %%	xml_header(-Header)
 %
@@ -234,10 +242,13 @@ page(Content) -->
 	html(html(Content)).
 
 page(Head, Body) -->
+	page(default, Head, Body).
+
+page(Style, Head, Body) -->
 	doctype,
 	html_begin(html),
-	pagehead(Head),
-	pagebody(Body),
+	pagehead(Style, Head),
+	pagebody(Style, Body),
 	html_end(html).
 
 %%	doctype//
@@ -256,35 +267,45 @@ doctype -->
 	[].
 
 
-pagehead(Head) -->
+pagehead(_, Head) -->
 	{ functor(Head, head, _)
 	}, !,
 	html(Head).
-pagehead(Head) -->
+pagehead(Style, Head) -->
 	{ strip_module(Head, M, _),
-	  hook_module(M, head//1)
-	}, !,
-	M:head(Head).
-pagehead(Head) -->
+	  hook_module(M, HM, head//2)
+	},
+	HM:head(Style, Head), !.
+pagehead(_, Head) -->
+	{ strip_module(Head, M, _),
+	  hook_module(M, HM, head//1)
+	},
+	HM:head(Head), !.
+pagehead(_, Head) -->
 	html(head(Head)).
 
 
-pagebody(Body) -->
+pagebody(_, Body) -->
 	{ functor(Body, body, _)
 	}, !,
 	html(Body).
-pagebody(Body) -->
+pagebody(Style, Body) -->
 	{ strip_module(Body, M, _),
-	  hook_module(M, body//1)
-	}, !,
-	M:body(Body).
-pagebody(Body) -->
+	  hook_module(M, HM, body//2)
+	},
+	HM:body(Style, Body), !.
+pagebody(_, Body) -->
+	{ strip_module(Body, M, _),
+	  hook_module(M, HM, body//1)
+	},
+	HM:body(Body), !.
+pagebody(_, Body) -->
 	html(body(Body)).
 
 
-hook_module(M, PI) :-
+hook_module(M, M, PI) :-
 	current_predicate(M:PI), !.
-hook_module(user, PI) :-
+hook_module(_, user, PI) :-
 	current_predicate(user:PI).
 
 %%	html(+Content:dom)// is det
@@ -321,10 +342,10 @@ do_expand(Fmt-Args, _) --> !,
 	{ format(string(String), Fmt, Args)
 	},
 	html_quoted(String).
-do_expand(\List, _) -->
+do_expand(\List, Module) -->
 	{ is_list(List)
 	}, !,
-	List.
+	raw(List, Module).
 do_expand(\Term, Module, In, Rest) :- !,
 	call(Module:Term, In, Rest).
 do_expand(Module:Term, _) --> !,
@@ -335,7 +356,9 @@ do_expand(script(Content), _) --> !,	% general CDATA declared content elements?
 	],
 	html_end(script).
 do_expand(&(Entity), _) --> !,
-	{ format(string(String), '&~w;', [Entity])
+	{   integer(Entity)
+	->  format(string(String), '&#~d;', [Entity])
+	;   format(string(String), '&~w;', [Entity])
 	},
 	[ String ].
 do_expand(Token, _) -->
@@ -385,6 +408,29 @@ check_non_empty(_, Tag, Term) :-
 	print_message(warning, format('Using empty element with content: ~p', [Term])).
 check_non_empty(_, _, _).
 
+%%	raw(+List, +Modules)// is det.
+%
+%	Emit unquoted (raw) output used for scripts, etc.
+
+raw([], _) -->
+	[].
+raw([H|T], Module) -->
+	raw_element(H, Module),
+	raw(T, Module).
+
+raw_element(Var, _) -->
+	{ var(Var), !,
+	  instantiation_error(Var)
+	}.
+raw_element(\Term, Module, In, Rest) :- !,
+	call(Module:Term, In, Rest).
+raw_element(Fmt-Args, _) --> !,
+	{ format(string(S), Fmt, Args) },
+	[S].
+raw_element(Value, _) -->
+	{ must_be(atomic, Value) },
+	[Value].
+
 
 %%	html_begin(+Env)// is det.
 %%	html_end(+End)// is det
@@ -394,7 +440,7 @@ check_non_empty(_, _, _).
 %	exceptional  cases.  Normal  applications    use   html//1.  The
 %	following two fragments are identical, where we prefer the first
 %	as it is more concise and less error-prone.
-%	
+%
 %	==
 %		html(table(border=1, \table_content))
 %	==
@@ -453,18 +499,18 @@ xhtml_empty(Env, Attributes) -->
 %	(http://www.w3.org/2006/07/SWD/RDFa/syntax/), embedding RDF   in
 %	(x)html provides a typical  usage  scenario   where  we  want to
 %	publish the required namespaces in the header. We can define:
-%	
+%
 %	==
 %	rdf_ns(Id) -->
 %		{ rdf_global_id(Id:'', Value) },
 %		xhtml_ns(Id, Value).
 %	==
-%	
+%
 %	After which we can use rdf_ns//1 as  a normal rule in html//1 to
 %	publish namespaces from library(semweb/rdf_db).   Note that this
 %	macro only has effect if  the  dialect   is  set  to =xhtml=. In
 %	=html= mode it is silently ignored.
-%	
+%
 %	The required =xmlns= receiver  is   installed  by  html_begin//1
 %	using the =html= tag and thus is   present  in any document that
 %	opens the outer =html= environment through this library.
@@ -525,21 +571,71 @@ name(NS:Name) --> !,
 name(Name) -->
 	[ Name ].
 
-%%	attribute_value(+Value)
+%%	attribute_value(+Value) is det.
 %
-%	Print an attribute value.  Value  is   either  atomic  or a term
-%	=|A+B|=, concatenating A and B.
+%	Print an attribute value. Value is either   atomic or one of the
+%	following terms:
+%
+%	  * A+B
+%	  Concatenation of A and B
+%	  * encode(V)
+%	  Emit URL-encoded version of V.  See www_form_encode/2.
+%	  * An option list
+%	  Emit ?Name1=encode(Value1)&Name2=encode(Value2) ...
+%
+%	The hook html_write:expand_attribute_value//1 can  be defined to
+%	provide additional `function like'   translations.  For example,
+%	http_dispatch.pl  defines  location_by_id(ID)  to   refer  to  a
+%	location on the current server  based   on  the  handler id. See
+%	http_location_by_id/2.
+
+:- multifile
+	expand_attribute_value//1.
 
 attribute_value(Var) -->
 	{ var(Var), !,
-	  throw(error(instantiation_error, _))
+	  instantiation_error(Var)
 	}.
 attribute_value(A+B) --> !,
 	attribute_value(A),
 	attribute_value(B).
+attribute_value([]) --> !.
+attribute_value(List) -->
+	{ is_list(List) }, !,
+	[ ? ],
+	search_parameters(List).
+attribute_value(encode(Value)) --> !,
+	{ www_form_encode(Value, Encoded) },
+	[ Encoded ].
+attribute_value(Value) -->
+	expand_attribute_value(Value), !.
 attribute_value(Value) -->
 	html_quoted_attribute(Value).
 
+search_parameters([H|T]) -->
+	search_parameter(H),
+	(   {T == []}
+	->  []
+	;   [&],
+	    search_parameters(T)
+	).
+
+search_parameter(Var) -->
+	{ var(Var), !,
+	  instantiation_error(Var)
+	}.
+search_parameter(Name=Value) -->
+	{ www_form_encode(Value, Encoded) },
+	[Name, =, Encoded].
+search_parameter(Term) -->
+	{ Term =.. [Name, Value], !,
+	  www_form_encode(Value, Encoded)
+	},
+	[Name, =, Encoded].
+search_parameter(Term) -->
+	{ domain_error(search_parameter, Term)
+	}.
+
 
 		 /*******************************
 		 *	   QUOTING RULES	*
@@ -551,11 +647,11 @@ attribute_value(Value) -->
 %	appearing in the document  structure   is  normally quoted using
 %	these rules. I.e. the following emits  properly quoted bold text
 %	regardless of the content of Text:
-%	
+%
 %	==
 %		html(b(Text))
 %	==
-%	
+%
 %	@tbd	Assumes UTF-8 encoding of the output.
 
 html_quoted(Text) -->
@@ -568,7 +664,7 @@ html_quoted(Text) -->
 %	included in double-quotes.  Note   that  -like  html_quoted//1-,
 %	attributed   values   printed   through   html//1   are   quoted
 %	atomatically.
-%	
+%
 %	@tbd	Assumes UTF-8 encoding of the output.
 
 html_quoted_attribute(Text) -->
@@ -584,13 +680,13 @@ html_quoted_attribute(Text) -->
 %
 %	Reposition HTML to  the  receiving   Id.  The  http_post//2 call
 %	processes HTML using html//1. Embedded   \-commands are executed
-%	by mainman/1 from  print_html/1   or  html_print_length/2. These
+%	by mailman/1 from  print_html/1   or  html_print_length/2. These
 %	commands are called in the calling   context of the html_post//2
 %	call.
-%	
+%
 %	A typical usage scenario is to  get   required  CSS links in the
 %	document head in a reusable fashion. First, we define css//1 as:
-%	
+%
 %	==
 %	css(URL) -->
 %		html_post(css,
@@ -599,10 +695,10 @@ html_quoted_attribute(Text) -->
 %				 href(URL)
 %			       ])).
 %	==
-%	
+%
 %	Next we insert the _unique_ CSS links, in the pagehead using the
 %	following call to reply_html_page/2:
-%	
+%
 %	==
 %		reply_html_page([ title(...),
 %				  \html_receive(css)
@@ -619,7 +715,7 @@ html_post(Id, Content) -->
 %	Receive posted HTML tokens. Unique   sequences  of tokens posted
 %	with  html_post//2  are  inserted   at    the   location   where
 %	html_receive//1 appears.
-%	
+%
 %	@see	The local predicate sorted_html//1 handles the output of
 %		html_receive//1.
 %	@see	html_receive//2 allows for post-processing the posted
@@ -634,7 +730,7 @@ html_receive(Id) -->
 %	called to process all messages posted to the channal at the time
 %	output is generated. Handler is a   grammar  rule that is called
 %	with three extra arguments.
-%	
+%
 %	    1. A list of Module:Term, of posted terms.  Module is the
 %	       contest module of html_post and Term is the unmodified
 %	       term.  Members are in the order posted and may contain
@@ -642,10 +738,10 @@ html_receive(Id) -->
 %	    2. DCG input list.  The final output must be produced by a
 %	       call to html//1.
 %	    3. DCG output list.
-%	    
+%
 %	Typically, Handler collects the posted   terms,  creating a term
 %	suitable for html//1 and finally calls html//1.
-    
+
 html_receive(Id, Handler) -->
 	{ strip_module(Handler, M, P) },
 	[ mailbox(Id, accept(M:P, _)) ].
@@ -694,7 +790,7 @@ mail_handlers([post(Module,HTML)|T0], H, [Module:HTML|T]) :- !,
 	mail_handlers(T0, H, T).
 mail_handlers([H|T0], [H|T], C) :-
 	mail_handlers(T0, T, C).
-	
+
 extend_args(Term, Extra, NewTerm) :-
 	Term =.. [Name|Args],
 	append(Args, [Extra], NewArgs),
@@ -704,7 +800,7 @@ extend_args(Term, Extra, NewTerm) :-
 %
 %	Default  handlers  for  html_receive//1.  It  sorts  the  posted
 %	objects to create a unique list.
-%	
+%
 %	@bug	Elements can differ just on the module.  Ideally we
 %		should phrase all members, sort the list of list of
 %		tokens and emit the result.  Can we do better?
@@ -718,7 +814,7 @@ sorted_html(List) -->
 %	Handler for html_receive(head). Unlike  sorted_html//1, it calls
 %	a user hook  html_write:html_head_expansion/2   to  process  the
 %	collected head material into a term suitable for html//1.
-%	
+%
 %	@tbd  This  has  been  added    to   facilate  html_head.pl,  an
 %	experimental  library  for  dealing  with   css  and  javascript
 %	resources. It feels a bit like a hack, but for now I do not know
@@ -778,14 +874,14 @@ post_close(_) -->
 %	Define required newlines before and after   tags.  This table is
 %	rather incomplete. New rules can  be   added  to  this multifile
 %	predicate.
-%	
+%
 %	@param Tag	Name of the tag
 %	@param Open	Tuple M-N, where M is the number of lines before
 %			the tag and N after.
 %	@param Close	Either as Open, or the atom - (minus) to imit the
 %			close-tag or =empty= to indicate the element has
 %			no content model.
-%			
+%
 % 	@tbd	Complete table
 
 :- multifile
@@ -796,7 +892,7 @@ layout(blockquote, 2-1,	1-2).
 layout(pre, 	   2-1,	1-2).
 layout(center,	   2-1,	1-2).
 layout(dl,	   2-1,	1-2).
-layout(ul,	   2-1,	1-2).
+layout(ul,	   1-1,	1-1).
 layout(ol,	   2-1,	1-2).
 layout(form,	   2-1,	1-2).
 layout(frameset,   2-1,	1-2).
@@ -852,7 +948,7 @@ layout(div,	   1-0,	0-1).
 %
 %		* nl(N)
 %		Use at minimum N newlines here.
-%		
+%
 %		* mailbox(Id, Box)
 %		Repositioned tokens (see html_post//2 and
 %		html_receive//2)
@@ -873,7 +969,8 @@ print_html(Out, List) :-
 	;   true
 	),
 	mailman(List),
-	write_html(List, Out).
+	write_html(List, Out),
+	flush_output(Out).
 
 write_html([], _).
 write_html([nl(N)|T], Out) :- !,
@@ -906,7 +1003,7 @@ write_nl(N, Out) :-
 %	Determine the content length of  a   token  list  produced using
 %	html//1. Here is an example on  how   this  is used to output an
 %	HTML compatible to HTTP:
-%	
+%
 %	==
 %		phrase(html(DOM), Tokens),
 %		html_print_length(Tokens, Len),
@@ -943,18 +1040,100 @@ html_print_length([H|T], L0, L) :-
 
 
 %%	reply_html_page(:Head, :Body) is det.
+%%	reply_html_page(+Style, :Head, :Body) is det.
 %
 %	Provide the complete reply as required  by http_wrapper.pl for a
 %	page constructed from Head and   Body. The HTTP =|Content-type|=
 %	is provided by html_current_option/1.
 
 reply_html_page(Head, Body) :-
+	reply_html_page(default, Head, Body).
+reply_html_page(Style, Head, Body) :-
 	html_current_option(content_type(Type)),
-	phrase(page(Head, Body), HTML),
+	phrase(page(Style, Head, Body), HTML),
 	format('Content-type: ~w~n~n', [Type]),
 	print_html(HTML).
 
 
+		 /*******************************
+		 *     META-PREDICATE SUPPORT	*
+		 *******************************/
+
+%%	html_meta(+Heads) is det.
+%
+%	This directive can be used  to   declare  that an HTML rendering
+%	rule takes HTML content as  argument.   It  has  two effects. It
+%	emits  the  appropriate  meta_predicate/1    and  instructs  the
+%	built-in editor (PceEmacs) to provide   proper colouring for the
+%	arguments.
+
+html_meta(Spec) :-
+	throw(error(context_error(nodirective, html_meta(Spec)), _)).
+
+html_meta_decls(Var, _, _) :-
+	var(Var), !,
+	instantiation_error(Var).
+html_meta_decls((A,B), (MA,MB), [MH|T]) :- !,
+	html_meta_decl(A, MA, MH),
+	html_meta_decls(B, MB, T).
+html_meta_decls(A, MA, MH) :-
+	html_meta_decl(A, MA, MH).
+
+html_meta_decl(Head, MetaHead,
+	       html_write:html_meta_head(GenHead, Module, Head)) :-
+	functor(Head, Name, Arity),
+	functor(GenHead, Name, Arity),
+	prolog_load_context(module, Module),
+	Head =.. [Name|HArgs],
+	maplist(html_meta_decl, HArgs, MArgs),
+	MetaHead =.. [Name|MArgs].
+
+html_meta_decl(html, :) :- !.
+html_meta_decl(Meta, Meta).
+
+system:term_expansion((:- html_meta(Heads)),
+		      [ (:- meta_predicate(Meta))
+		      | MetaHeads
+		      ]) :-
+	html_meta_decls(Heads, Meta, MetaHeads).
+
+:- multifile
+	html_meta_head/3.
+
+html_meta_colours(Head, Goal, built_in-Colours) :-
+	Head =.. [_|MArgs],
+	Goal =.. [_|Args],
+	maplist(meta_colours, MArgs, Args, Colours).
+
+meta_colours(html, HTML, Colours) :- !,
+	html_colours(HTML, Colours).
+meta_colours(_, _, classify).
+
+html_meta_called(Head, Goal, Called) :-
+	Head =.. [_|MArgs],
+	Goal =.. [_|Args],
+	meta_called(MArgs, Args, Called, []).
+
+meta_called([], [], Called, Called).
+meta_called([html|MT], [A|AT], Called, Tail) :- !,
+	phrase(called_by(A), Called, Tail1),
+	meta_called(MT, AT, Tail1, Tail).
+meta_called([_|MT], [_|AT], Called, Tail) :- !,
+	meta_called(MT, AT, Called, Tail).
+
+
+:- html_meta
+	html(html,?,?),
+	page(html,?,?),
+	page(html,html,?,?),
+	page(html,html,?,?),
+	pagehead(html,?,?),
+	pagebody(html,?,?),
+	reply_html_page(html,html),
+	reply_html_page(+,html,html),
+	html_post(+,html,?,?).
+
+
 		 /*******************************
 		 *	PCE EMACS SUPPORT	*
 		 *******************************/
@@ -965,36 +1144,17 @@ reply_html_page(Head, Body) :-
 	emacs_prolog_colours:identify/2,
 	prolog:called_by/2.
 
-emacs_prolog_colours:goal_colours(html(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(page(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(page(Head, Body,_,_),
-				  built_in-[HC, BC, classify, classify]) :-
-	html_colours(Head, HC),
-	html_colours(Body, BC).
-emacs_prolog_colours:goal_colours(pagehead(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(pagebody(HTML,_,_),
-				  built_in-[Colours, classify, classify]) :-
-	html_colours(HTML, Colours).
-emacs_prolog_colours:goal_colours(reply_html_page(Head, Body),
-				  built_in-[HC, BC]) :-
-	html_colours(Head, HC),
-	html_colours(Body, BC).
-emacs_prolog_colours:goal_colours(html_post(_Id, HTML, _, _),
-				  built_in-[classify, Colours]) :-
-	html_colours(HTML, Colours).
+emacs_prolog_colours:goal_colours(Goal, Colours) :-
+	html_meta_head(Goal, _Module, Head),
+	html_meta_colours(Head, Goal, Colours).
 
 
 					% TBD: Check with do_expand!
 html_colours(Var, classify) :-
 	var(Var), !.
-html_colours(\List, classify) :-
-	is_list(List), !.
+html_colours(\List, built_in-Colours) :-
+	is_list(List), !,
+	list_colours(List, Colours).
 html_colours(\_, built_in-[dcg]) :- !.
 html_colours(_:Term, built_in-[classify,Colours]) :- !,
 	html_colours(Term, Colours).
@@ -1035,13 +1195,15 @@ attr_colours([], classify) :- !.
 attr_colours(Term, list-Elements) :-
 	Term = [_|_], !,
 	attr_list_colours(Term, Elements).
-attr_colours(Name=_, built_in-[html_attribute(Name), classify]) :- !.
+attr_colours(Name=Value, built_in-[html_attribute(Name), VColour]) :- !,
+	attr_value_colour(Value, VColour).
 attr_colours(NS:Term, built_in-[html_xmlns(NS), html_attribute(Name)-[classify]]) :-
 	compound(Term),
 	Term =.. [Name,_], !.
-attr_colours(Term, html_attribute(Name)-[classify]) :-
+attr_colours(Term, html_attribute(Name)-[VColour]) :-
 	compound(Term),
-	Term =.. [Name,_], !.
+	Term =.. [Name,Value], !,
+	attr_value_colour(Value, VColour).
 attr_colours(Name, html_attribute(Name)) :-
 	atom(Name), !.
 attr_colours(_, error).
@@ -1053,6 +1215,30 @@ attr_list_colours([H0|T0], [H|T]) :-
 	attr_colours(H0, H),
 	attr_list_colours(T0, T).
 
+attr_value_colour(Var, classify) :-
+	var(Var).
+attr_value_colour(location_by_id(ID), sgml_attr_function-[Colour]) :- !,
+	location_id(ID, Colour).
+attr_value_colour(A+B, sgml_attr_function-[CA,CB]) :- !,
+	attr_value_colour(A, CA),
+	attr_value_colour(B, CB).
+attr_value_colour(encode(_), sgml_attr_function-[classify]) :- !.
+attr_value_colour(Atom, classify) :-
+	atomic(Atom), !.
+attr_value_colour(List, classify) :-
+	is_list(List), !.
+attr_value_colour(_, error).
+
+location_id(ID, classify) :-
+	var(ID), !.
+location_id(ID, Class) :-
+	current_predicate(http_dispatch:http_location_by_id/2),
+	(   catch(http_dispatch:http_location_by_id(ID, Location), _, fail)
+	->  Class = http_location_for_id(Location)
+	;   Class = http_no_location_for_id(ID)
+	).
+location_id(_, classify).
+
 
 :- op(990, xfx, :=).			% allow compiling without XPCE
 :- op(200, fy, @).
@@ -1062,6 +1248,9 @@ emacs_prolog_colours:style(html(_), style(bold := @on,
 emacs_prolog_colours:style(entity(_), style(colour := magenta4)).
 emacs_prolog_colours:style(html_attribute(_), style(colour := magenta4)).
 emacs_prolog_colours:style(html_xmlns(_), style(colour := magenta4)).
+emacs_prolog_colours:style(sgml_attr_function, style(colour := blue)).
+emacs_prolog_colours:style(http_location_for_id(_), style(bold := @on)).
+emacs_prolog_colours:style(http_no_location_for_id(_), style(colour := red, bold := @on)).
 
 
 emacs_prolog_colours:identify(html(Element), Summary) :-
@@ -1070,28 +1259,22 @@ emacs_prolog_colours:identify(entity(Entity), Summary) :-
 	format(string(Summary), '~w: SGML entity', [Entity]).
 emacs_prolog_colours:identify(html_attribute(Attr), Summary) :-
 	format(string(Summary), '~w: SGML attribute', [Attr]).
+emacs_prolog_colours:identify(sgml_attr_function, 'SGML Attribute function').
+emacs_prolog_colours:identify(http_location_for_id(Location), Summary) :-
+	format(string(Summary), 'ID resolves to ~w', [Location]).
+emacs_prolog_colours:identify(http_no_location_for_id(ID), Summary) :-
+	format(string(Summary), '~w: no such ID', [ID]).
 
 
 %	prolog:called_by(+Goal, -Called)
-%	
+%
 %	Hook into library(pce_prolog_xref).  Called is a list of callable
 %	or callable+N to indicate (DCG) arglist extension.
 
 
-prolog:called_by(html(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(page(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(page(Head,Body,_,_), Called) :-
-	phrase(called_by([Head,Body]), Called).
-prolog:called_by(pagehead(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(pagebody(HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(html_post(_,HTML,_,_), Called) :-
-	phrase(called_by(HTML), Called).
-prolog:called_by(reply_html_page(Head,Body), Called) :-
-	phrase(called_by([Head,Body]), Called).
+prolog:called_by(Goal, Called) :-
+	html_meta_head(Goal, _Module, Head),
+	html_meta_called(Head, Goal, Called).
 
 called_by(Term) -->
 	called_by(Term, _).
@@ -1101,9 +1284,9 @@ called_by(Var, _) -->
 	[].
 called_by(\G, M) --> !,
 	(   { is_list(G) }
-	->  []
+	->  called_by(G, M)
 	;   {atom(M)}
-	->  [M:G+2]
+	->  [(M:G)+2]
 	;   [G+2]
 	).
 called_by([], _) --> !,
@@ -1116,7 +1299,7 @@ called_by(M:Term, _) --> !,
 	->  called_by(Term, M)
 	;   []
 	).
-called_by(Term, M) --> 
+called_by(Term, M) -->
 	{ compound(Term), !,
 	  Term =.. [_|Args]
 	},
