@@ -57,8 +57,9 @@
 	]).
 
 
-variable(comment_column,	int,	both, "Column for line comment").
-variable(show_line_numbers,	[bool], get,  "Show line numbers?").
+variable(comment_column,    int,        both, "Column for line comment").
+variable(show_line_numbers, 'int|bool', get,  "Show line numbers?").
+class_variable(show_line_numbers, 'int|bool', 250000).
 
 setup_mode(E) :->
 	"Switch editor into fill-mode"::
@@ -69,7 +70,6 @@ setup_mode(E) :->
 initialise(M) :->
 	"Inititialise comment_column"::
 	send(M, send_super, initialise),
-	send(M, slot, show_line_numbers, @default),
 	send(M, comment_column, @emacs_comment_column).
 
 
@@ -136,7 +136,7 @@ comment_lines(TB, S0, End, Comment) :-
 	->  comment_lines(TB, S2, End1, Comment)
 	;   true
 	).
-	
+
 
 fill_comment_paragraph(M, Justify:justify=[bool|int], From:[int]) :->
 	"Fill paragraph in (line) comment"::
@@ -146,7 +146,7 @@ fill_comment_paragraph(M, Justify:justify=[bool|int], From:[int]) :->
 	    fail
 	),
 	new(Re, regex(string('^%s?[ \t]*$', CS))),
-	new(LeadRe, regex(string('%s[ \t]*', CS))),
+	new(LeadRe, regex(string('%s%s*[ \t]*', CS, CS))),
 	get(M, caret, Caret),
 	get(M, text_buffer, TB),
 	(   From \== @default
@@ -192,7 +192,7 @@ fill_comment(M,
 	send(M, caret, CaretF?start),
 	free(EndF),
 	free(CaretF).
-	
+
 uncomment(_M, _Re, _LeadCol, Here, EndF) :-
 	get(EndF, start, End),
 	Here >= End, !.
@@ -231,7 +231,7 @@ insert_comment_block(E) :->
 
 insert_section_header(E) :->
 	"Insert Prolog/C section header"::
-	send(E, insert, 
+	send(E, insert,
 '		 /*******************************
 		 *               C		*
 		 *******************************/
@@ -305,7 +305,7 @@ file_header_parameter('%E', _, Email) :-
 	->  true
 	;   get(@pce, user, User),
 	    gethostname(Host),
-	    concat_atom([User, Host], @, Email)
+	    atomic_list_concat([User, Host], @, Email)
 	).
 file_header_parameter('%D', _, Date) :-
 	new(D, date),
@@ -422,7 +422,7 @@ indent_expression_line(E, Brackets:[name], Base:[int]) :->
 	    ),
 	    send(E, align_line, Col), !.
 
-		
+
 %	->indent_comment_line
 %
 
@@ -561,12 +561,12 @@ expand_tag(M, Tag:[name], TheTag:name) :<-
 	).
 
 
-find_tag(M, Tag:emacs_tag, Editor:editor) :<-
+find_tag(M, Tag:emacs_tag, Where:[{here,tab,window}], Editor:editor) :<-
 	"Jump to indicated tag entry"::
 	(   auto_call(emacs_tag(Tag, File, Line)),
 	    new(B, emacs_buffer(File)),
-	    send(B, open),
-	    get(B?editors, head, Editor),
+	    get(B, open, Where, Frame),
+	    get(Frame, editor, Editor),
 	    send(Editor, line_number, Line),
 	    adjust_tag(Editor, Tag)
 	;   send(M, report, warning, 'Cannot find tag %s', Tag),
@@ -576,7 +576,7 @@ find_tag(M, Tag:emacs_tag, Editor:editor) :<-
 
 find_tag(M, Tag:emacs_tag) :->
 	"Jump to entry from TAG table"::
-	ignore(get(M, find_tag, Tag, _)). % avoid delegation to menu-bar
+	ignore(get(M, find_tag, Tag, tab, _)). % avoid delegation to menu-bar
 
 
 adjust_tag(E, Tag) :-
@@ -631,10 +631,11 @@ new_caret_position(M, Caret:int) :->
 	send_super(M, new_caret_position, Caret),
 	(   get(M, frame, Frame),
 	    send(Frame, has_send_method, show_line_number)
-	->  (	get(M, show_line_numbers, @off)
+	->  get(M, show_line_numbers, How),
+	    (	How == @off
 	    ->	send(Frame, show_line_number, @nil)
-	    ;   (   (   get(M, show_line_numbers, @default)
-		    ->  Caret < 50000
+	    ;   (   (   integer(How)
+		    ->  Caret =< How
 		    ;   get(M, show_line_numbers, @on)
 		    )
 		->  get(M, line_number, Line),
