@@ -1,3 +1,36 @@
+/*  $Id$
+
+    Part of XPCE --- The SWI-Prolog GUI toolkit
+
+    Author:        Jan Wielemaker
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org/packages/xpce/
+    Copyright (C): 2003-2010, University of Amsterdam
+			      VU University Amsterdam
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
+*/
+
+
 :- module(pce_profile,
 	  [ pce_show_profile/0
 	  ]).
@@ -9,8 +42,14 @@
 :- use_module(library(tabular)).
 :- use_module(library(prolog_predicate)).
 
-%	pce_show_profile/0
-%	
+/** <module> GUI frontend for the profiler
+
+This module hooks into profile/1 and  provides   a  graphical UI for the
+profiler output.
+*/
+
+%%	pce_show_profile/0
+%
 %	Show already collected profile using a graphical browser.
 
 pce_show_profile :-
@@ -75,7 +114,7 @@ fill_dialog(F, TD:tool_dialog) :->
 		    menu_item(help,
 			      message(F, help))
 		  ]).
-			      
+
 
 load_profile(F, ProfData:[prolog]) :->
 	"Load stored profile from the Prolog database"::
@@ -129,7 +168,7 @@ details(F, From:prolog) :->
 	->  get(DI, data, Node),
 	    send(W, node, Node)
 	).
-	    
+
 sort_by(F, SortBy:name, Order:[{normal,reverse}]) :->
 	"Define the key for sorting the flat profile"::
 	get(F, member, prof_browser, B),
@@ -209,7 +248,7 @@ load_profile(B, Nodes:prolog) :->
 	forall(member(Node, Nodes),
 	       send(B, append, prof_dict_item(Node, SortBy, Frame))),
 	send(B, sort).
-	
+
 update_label(B) :->
 	get(B, sort_by, Sort),
 	sort_by(Human, Sort, _How),
@@ -250,7 +289,7 @@ initialise(DI, Node:prolog, SortBy:name, F:prof_frame) :->
 	predicate_label(Predicate, Key),
 	send_super(DI, initialise, Key),
 	send(DI, update_label, SortBy, F).
-	
+
 value(DI, Name:name, Value:prolog) :<-
 	"Get associated value"::
 	get(DI, data, Data),
@@ -291,7 +330,7 @@ update_label(DI, SortBy:name, F:prof_frame) :->
 
 time_key(ticks).
 time_key(ticks_self).
-time_key(ticks_siblings).
+time_key(ticks_children).
 
 details(DI) :->
 	"Show details"::
@@ -338,7 +377,7 @@ title(W) :->
 	     rowspan := 2),
 	send(T, next_row),
 	send(T, append, 'Self',   bold, center, BG),
-	send(T, append, 'Siblings',   bold, center, BG),
+	send(T, append, 'Children',   bold, center, BG),
 	send(T, append, 'Call',   bold, center, BG),
 	send(T, append, 'Redo',   bold, center, BG),
 	send(T, next_row).
@@ -395,14 +434,14 @@ show_cluster(Callers, Callees, Data, Cycle, W) :-
 	),
 	sort_relatives(Callers, Callers1),
 	show_relatives(Callers1, parent, W),
-	ticks(Callers1, Self, Siblings, Call, Redo),
-	send(W, show_predicate, Data, Self, Siblings, Call, Redo),
+	ticks(Callers1, Self, Children, Call, Redo),
+	send(W, show_predicate, Data, Self, Children, Call, Redo),
 	sort_relatives(Callees, Callees1),
 	reverse(Callees1, Callees2),
-	show_relatives(Callees2, sibling, W).
+	show_relatives(Callees2, child, W).
 
-ticks(Callers, Self, Siblings, Call, Redo) :-
-	ticks(Callers, 0, Self, 0, Siblings, 0, Call, 0, Redo).
+ticks(Callers, Self, Children, Call, Redo) :-
+	ticks(Callers, 0, Self, 0, Children, 0, Call, 0, Redo).
 
 ticks([], Self, Self, Sibl, Sibl, Call, Call, Redo, Redo).
 ticks([H|T],
@@ -422,7 +461,7 @@ ticks([H|T], Self0, Self, Sibl0, Sibl, Call0, Call, Redo0, Redo) :-
 
 
 %	clusters(+Relatives, -Cycles)
-%	
+%
 %	Organise the relatives by cluster.
 
 clusters(Relatives, Cycles) :-
@@ -442,7 +481,7 @@ cluster([H|T0], C, TC, [H|T]) :-
 	cluster(T0, C, TC, T).
 
 %	sort_relatives(+Relatives, -Sorted)
-%	
+%
 %	Sort relatives in ascending number of calls.
 
 sort_relatives(List, Sorted) :-
@@ -465,7 +504,7 @@ unkey([_-H|T0], [H|T]) :-
 	unkey(T0, T).
 
 %	show_relatives(+Relatives, +Rolw, +Window)
-%	
+%
 %	Show list of relatives as table-rows.
 
 show_relatives([], _, _) :- !.
@@ -474,17 +513,17 @@ show_relatives([H|T], Role, W) :-
 	show_relatives(T, Role, W).
 
 show_predicate(W, Data:prolog,
-	       Ticks:int, SiblingTicks:int,
+	       Ticks:int, ChildTicks:int,
 	       Call:int, Redo:int) :->
 	"Show the predicate we have details on"::
 	value(Data, predicate, Pred),
 	get(W, frame, Frame),
 	get(Frame, render_time, Ticks, Self),
-	get(Frame, render_time, SiblingTicks, Siblings),
+	get(Frame, render_time, ChildTicks, Children),
 	get(W, tabular, T),
 	BG = (background := khaki1),
 	send(T, append, Self, halign := right, BG),
-	send(T, append, Siblings, halign := right, BG),
+	send(T, append, Children, halign := right, BG),
 	send(T, append, Call, halign := right, BG),
 	send(T, append, Redo, halign := right, BG),
 	(   object(Pred)
@@ -496,7 +535,7 @@ show_predicate(W, Data:prolog,
 	send(T, next_row).
 
 show_relative(W, Caller:prolog, Role:name) :->
-	Caller = node(Pred, _Cluster, Ticks, SiblingTicks, Calls, Redos),
+	Caller = node(Pred, _Cluster, Ticks, ChildTicks, Calls, Redos),
 	get(W, tabular, T),
 	get(W, frame, Frame),
 	(   Pred == '<recursive>'
@@ -505,9 +544,9 @@ show_relative(W, Caller:prolog, Role:name) :->
 	    send(T, append, new(graphical)),
 	    send(T, append, Pred, italic)
 	;   get(Frame, render_time, Ticks, Self),
-	    get(Frame, render_time, SiblingTicks, Siblings),
+	    get(Frame, render_time, ChildTicks, Children),
 	    send(T, append, Self, halign := right),
-	    send(T, append, Siblings, halign := right),
+	    send(T, append, Children, halign := right),
 	    send(T, append, Calls, halign := right),
 	    send(T, append, Redos, halign := right),
 	    (   Pred == '<spontaneous>'
@@ -527,9 +566,9 @@ show_relative(W, Caller:prolog, Role:name) :->
 		   "Show executable object").
 
 variable(context,   any,		   get, "Represented executable").
-variable(role,	    {parent,self,sibling}, get,	"Represented role").
+variable(role,	    {parent,self,child}, get,	"Represented role").
 
-initialise(T, Context:any, Role:{parent,self,sibling}, Cycle:[int]) :->
+initialise(T, Context:any, Role:{parent,self,child}, Cycle:[int]) :->
 	send(T, slot, context, Context),
 	send(T, slot, role, Role),
 	get(T, label, Label),
@@ -603,7 +642,7 @@ details(T) :->
 :- pce_begin_class(prof_predicate_text, prof_node_text,
 		   "Show a predicate").
 
-initialise(T, Pred:prolog, Role:{parent,self,sibling}, Cycle:[int]) :->
+initialise(T, Pred:prolog, Role:{parent,self,child}, Cycle:[int]) :->
 	send_super(T, initialise, prolog_predicate(Pred), Role, Cycle).
 
 details(T) :->
@@ -620,7 +659,7 @@ details(T) :->
 
 key(predicate,	    1).
 key(ticks_self,	    2).
-key(ticks_siblings, 3).
+key(ticks_children, 3).
 key(call,	    4).
 key(redo,	    5).
 key(callers,	    6).
@@ -634,8 +673,8 @@ value(Data, label, Label) :- !,
 	predicate_label(Pred, Label).
 value(Data, ticks, Ticks) :- !,
 	arg(2, Data, Self),
-	arg(3, Data, Siblings),
-	Ticks is Self + Siblings.
+	arg(3, Data, Children),
+	Ticks is Self + Children.
 value(Data, Name, Value) :-
 	key(Name, Arg),
 	arg(Arg, Data, Value).
@@ -643,27 +682,33 @@ value(Data, Name, Value) :-
 sort_by(flat_profile_by_name,		     name,	     normal).
 sort_by(cumulative_profile_by_time,	     ticks,	     reverse).
 sort_by(flat_profile_by_time_self,	     ticks_self,     reverse).
-sort_by(cumulative_profile_by_time_siblings, ticks_siblings, reverse).
+sort_by(cumulative_profile_by_time_children, ticks_children, reverse).
 sort_by(flat_profile_by_number_of_calls,     call,	     reverse).
 sort_by(flat_profile_by_number_of_redos,     redo,	     reverse).
 
 
-%	predicate_label(+Head, -Label)
-%	
-%	Create a human-readable label for the given head
+%%	predicate_label(+Head, -Label)
+%
+%	Label is the human-readable identification   for Head. Calls the
+%	hook user:prolog_predicate_name/2.
+
+:- multifile
+	user:prolog_predicate_name/2.
 
 predicate_label(Obj, Label) :-
 	object(Obj), !,
 	get(Obj, print_name, Label).
+predicate_label(Head, Label) :-
+	user:prolog_predicate_name(Head, Label), !.
 predicate_label(M:H, Label) :- !,
 	functor(H, Name, Arity),
 	(   hidden_module(M, H)
-	->  concat_atom([Name, /, Arity], Label)
-	;   concat_atom([M, :, Name, /, Arity], Label)
+	->  atomic_list_concat([Name, /, Arity], Label)
+	;   atomic_list_concat([M, :, Name, /, Arity], Label)
 	).
 predicate_label(H, Label) :- !,
 	functor(H, Name, Arity),
-	concat_atom([Name, /, Arity], Label).
+	atomic_list_concat([Name, /, Arity], Label).
 
 hidden_module(system, _).
 hidden_module(user, _).
@@ -675,24 +720,26 @@ predicate_name(_:H, Name) :- !,
 predicate_name(H, Name) :-
 	functor(H, Name, _Arity).
 
-%	prof_node(node(Pred,
-%		       TimeSelf, TimeSiblings,
-%		       Calls, Redo, Recursive,
-%		       Parents))
+%%	prof_node(Data:node(Pred,
+%%		            TicksSelf, TicksChildren,
+%%		            Calls, Redo,
+%%		            Parents, Children)) is nondet.
 %
-%	Collect data for each of the interesting predicates.
+%	Data is a node of the   profile call-tree. Backtracking over all
+%	solutions  generates  the   entire    call-tree   with   profile
+%	statistics.
 
 prof_node(node(Impl,
-	       TicksSelf, TicksSiblings,
+	       TicksSelf, TicksChildren,
 	       Call, Redo,
-	       Parents, Siblings)) :-
+	       Parents, Children)) :-
 	setof(Impl, prof_impl(Impl, -), Impls0),
 	join_impl(Impls0, Impls),
 	member(Impl, Impls),
 	'$prof_procedure_data'(Impl,
-			       TicksSelf, TicksSiblings,
+			       TicksSelf, TicksChildren,
 			       Call, Redo,
-			       Parents, Siblings).
+			       Parents, Children).
 
 join_impl([], []).
 join_impl([H|T0], [H|T]) :-
