@@ -30,7 +30,7 @@
 */
 
 :- module(pce_thread_monitor,
-	  [ 
+	  [
 	  ]).
 :- use_module(library(pce)).
 :- use_module(library(toolbar)).
@@ -58,22 +58,26 @@ resource(profiling, image, image('16x16/profiler.xpm')).
 :- pce_begin_class(thread_statistics, object,
 		   "Record historic status of a thread").
 
-variable(local,	 int, get, "Local-stack usage").
-variable(global, int, get, "Global-stack usage").
-variable(trail,	 int, get, "Trail-stack usage").
-variable(cpu,	 int, get, "%CPU (0..100)").
+variable(local,	 int := 0, get, "Local-stack usage").
+variable(global, int := 0, get, "Global-stack usage").
+variable(trail,	 int := 0, get, "Trail-stack usage").
+variable(cpu,	 int := 0, get, "%CPU (0..100)").
 
 initialise(TS, Thread:any, CPU:int) :->
 	"Create for victim"::
 	send_super(TS, initialise),
+	(   catch(thread_stack_sizes(Thread, Local, Global, Trail), _, fail)
+	->  send(TS, slot, local,  Local),
+	    send(TS, slot, global, Global),
+	    send(TS, slot, trail,  Trail)
+	;   true
+	),
+	send(TS, slot, cpu, CPU).
+
+thread_stack_sizes(Thread, Local, Global, Trail) :-
 	thread_statistics(Thread, localused, Local),
 	thread_statistics(Thread, globalused, Global),
-	thread_statistics(Thread, trailused, Trail),
-	
-	send(TS, slot, local,  Local),
-	send(TS, slot, global, Global),
-	send(TS, slot, trail,  Trail),
-	send(TS, slot, cpu,    CPU).
+	thread_statistics(Thread, trailused, Trail).
 
 :- pce_end_class(thread_statistics).
 
@@ -124,7 +128,12 @@ update(TS, Status:prolog, CpuH:[int]) :->
 
 cpu_percentage(TS, CPU:'0..100') :<-
 	get(TS, key, TID),
-	thread_statistics(TID, cputime, Now),
+	(   catch(thread_statistics(TID, cputime, Now), _, fail)
+	->  true
+	;   get(TS, lastcpu, Now), Now \== @nil
+	->  true
+	;   Now = 0.0
+	),
 	get_time(WallNow),
 	(   get(TS, lastcpu, Last), Last \== @nil,
 	    get(TS, lastwall, LastWall),
@@ -159,7 +168,7 @@ truncate_chain(Chain, Len) :-
 truncate_chain(Chain, Len) :-
 	send(Chain, delete_head),
 	truncate_chain(Chain, Len).
-	
+
 
 join(TS) :->
 	"Join this thread"::
@@ -177,7 +186,7 @@ start_profile(TS) :->
 	(   get(TS, profiling, @on)
 	->  send(TS, report, warning, 'Already profiling')
 	;   get(TS, key, TID),
-	    thread_signal(TID, 
+	    thread_signal(TID,
 			  (	  reset_profiler,
 				  profiler(_, true)
 			  )),
@@ -190,7 +199,7 @@ end_profile(TS) :->
 	(   get(TS, profiling, @off)
 	->  send(TS, report, error, 'Not profiling')
 	;   get(TS, key, TID),
-	    thread_signal(TID, 
+	    thread_signal(TID,
 			  (	  profiler(_, false),
 				  show_profile(plain, 25)
 			  )),
@@ -391,7 +400,7 @@ initialise(TD, TS:thread_status, Graphs:chain, H:int, V:int) :->
 	new(_, partof_hyper(TS, TD, diagram, status)),
 	send(TS, recall, H),
 	send(TD, update).
-	
+
 
 attach(TD, TS:thread_status) :->
 	"Switch to another thread"::
@@ -512,7 +521,7 @@ label_for_value(A, Val:'int|real', Gr:graphical) :<-
 	),
 	get(A, tag_font, Font),
 	new(Gr, text(S, right, Font)).
-	
+
 location(A, Sel:name, Val:'int|real', Loc:int) :<-
 	"Ignore values < low"::
 	(   percent_scale(Sel)
